# Stubs for requests.utils (Python 3)

from typing import Any
from . import compat
from . import cookies
from . import structures
from . import exceptions

OrderedDict = compat.OrderedDict
RequestsCookieJar = cookies.RequestsCookieJar
cookiejar_from_dict = cookies.cookiejar_from_dict
CaseInsensitiveDict = structures.CaseInsensitiveDict
InvalidURL = exceptions.InvalidURL

NETRC_FILES = ...  # type: Any
DEFAULT_CA_BUNDLE_PATH = ...  # type: Any

def dict_to_sequence(d): ...
def super_len(o): ...
def get_netrc_auth(url): ...
def guess_filename(obj): ...
def from_key_val_list(value): ...
def to_key_val_list(value): ...
def parse_list_header(value): ...
def parse_dict_header(value): ...
def unquote_header_value(value, is_filename=...): ...
def dict_from_cookiejar(cj): ...
def add_dict_to_cookiejar(cj, cookie_dict): ...
def get_encodings_from_content(content): ...
def get_encoding_from_headers(headers): ...
def stream_decode_response_unicode(iterator, r): ...
def iter_slices(string, slice_length): ...
def get_unicode_from_response(r): ...

UNRESERVED_SET = ...  # type: Any

def unquote_unreserved(uri): ...
def requote_uri(uri): ...
def address_in_network(ip, net): ...
def dotted_netmask(mask): ...
def is_ipv4_address(string_ip): ...
def is_valid_cidr(string_network): ...
def set_environ(env_name, value): ...
def should_bypass_proxies(url): ...
def get_environ_proxies(url): ...
def default_user_agent(name=...): ...
def default_headers(): ...
def parse_header_links(value): ...
def guess_json_utf(data): ...
def prepend_scheme_if_needed(url, new_scheme): ...
def get_auth_from_url(url): ...
def to_native_string(string, encoding=...): ...
def urldefragauth(url): ...



from typing import Any, Optional
from wsgiref.types import WSGIEnvironment, InputStream

def responder(f): ...
def get_current_url(environ, root_only=False, strip_querystring=False, host_only=False, trusted_hosts=None): ...
def host_is_trusted(hostname, trusted_list): ...
def get_host(environ, trusted_hosts=None): ...
def get_content_length(environ: WSGIEnvironment) -> Optional[int]: ...
def get_input_stream(environ: WSGIEnvironment, safe_fallback: bool = ...) -> InputStream: ...
def get_query_string(environ): ...
def get_path_info(environ, charset='', errors=''): ...
def get_script_name(environ, charset='', errors=''): ...
def pop_path_info(environ, charset='', errors=''): ...
def peek_path_info(environ, charset='', errors=''): ...
def extract_path_info(environ_or_baseurl, path_or_url, charset='', errors='', collapse_http_schemes=True): ...

class SharedDataMiddleware:
    app = ...  # type: Any
    exports = ...  # type: Any
    cache = ...  # type: Any
    cache_timeout = ...  # type: Any
    fallback_mimetype = ...  # type: Any
    def __init__(self, app, exports, disallow=None, cache=True, cache_timeout=..., fallback_mimetype=''): ...
    def is_allowed(self, filename): ...
    def get_file_loader(self, filename): ...
    def get_package_loader(self, package, package_path): ...
    def get_directory_loader(self, directory): ...
    def generate_etag(self, mtime, file_size, real_filename): ...
    def __call__(self, environ, start_response): ...

class DispatcherMiddleware:
    app = ...  # type: Any
    mounts = ...  # type: Any
    def __init__(self, app, mounts=None): ...
    def __call__(self, environ, start_response): ...

class ClosingIterator:
    def __init__(self, iterable, callbacks=None): ...
    def __iter__(self): ...
    def __next__(self): ...
    def close(self): ...

def wrap_file(environ, file, buffer_size=8192): ...

class FileWrapper:
    file = ...  # type: Any
    buffer_size = ...  # type: Any
    def __init__(self, file, buffer_size=8192): ...
    def close(self): ...
    def seekable(self): ...
    def seek(self, *args): ...
    def tell(self): ...
    def __iter__(self): ...
    def __next__(self): ...

class _RangeWrapper:
    iterable = ...  # type: Any
    byte_range = ...  # type: Any
    start_byte = ...  # type: Any
    end_byte = ...  # type: Any
    read_length = ...  # type: Any
    seekable = ...  # type: Any
    end_reached = ...  # type: Any
    def __init__(self, iterable, start_byte=0, byte_range=None): ...
    def __iter__(self): ...
    def __next__(self): ...
    def close(self): ...

def make_line_iter(stream, limit=None, buffer_size=..., cap_at_buffer=False): ...
def make_chunk_iter(stream, separator, limit=None, buffer_size=..., cap_at_buffer=False): ...

class LimitedStream:
    limit = ...  # type: Any
    def __init__(self, stream, limit): ...
    def __iter__(self): ...
    @property
    def is_exhausted(self): ...
    def on_exhausted(self): ...
    def on_disconnect(self): ...
    def exhaust(self, chunk_size=...): ...
    def read(self, size=None): ...
    def readline(self, size=None): ...
    def readlines(self, size=None): ...
    def tell(self): ...
    def __next__(self): ...



from typing import List
import sys

def exists(path: str) -> bool: ...
def isfile(path: str) -> bool: ...
def isdir(s: str) -> bool: ...
def getsize(filename: str) -> int: ...
def getmtime(filename: str) -> float: ...
def getatime(filename: str) -> float: ...
def getctime(filename: str) -> float: ...
def commonprefix(m: List[str]) -> str: ...

if sys.version_info >= (3, 4):
    def samestat(s1: str, s2: str) -> int: ...
    def samefile(f1: str, f2: str) -> int: ...
    def sameopenfile(fp1: str, fp2: str) -> int: ...



from typing import Any, IO, Text
import io

class GzipFile(io.BufferedIOBase):
    myfileobj = ...  # type: Any
    max_read_chunk = ...  # type: Any
    mode = ...  # type: Any
    extrabuf = ...  # type: Any
    extrasize = ...  # type: Any
    extrastart = ...  # type: Any
    name = ...  # type: Any
    min_readsize = ...  # type: Any
    compress = ...  # type: Any
    fileobj = ...  # type: Any
    offset = ...  # type: Any
    mtime = ...  # type: Any
    def __init__(self, filename: str = ..., mode: Text = ..., compresslevel: int = ...,
                 fileobj: IO[str] = ..., mtime: float = ...) -> None: ...
    @property
    def filename(self): ...
    size = ...  # type: Any
    crc = ...  # type: Any
    def write(self, data): ...
    def read(self, size=...): ...
    @property
    def closed(self): ...
    def close(self): ...
    def flush(self, zlib_mode=...): ...
    def fileno(self): ...
    def rewind(self): ...
    def readable(self): ...
    def writable(self): ...
    def seekable(self): ...
    def seek(self, offset, whence=...): ...
    def readline(self, size=...): ...

def open(filename: str, mode: Text = ..., compresslevel: int = ...) -> GzipFile: ...



from typing import Any, Dict, List, Mapping, Optional, Type
from boto.connection import AWSQueryConnection

class KMSConnection(AWSQueryConnection):
    APIVersion = ...  # type: str
    DefaultRegionName = ...  # type: str
    DefaultRegionEndpoint = ...  # type: str
    ServiceName = ...  # type: str
    TargetPrefix = ...  # type: str
    ResponseError = ...  # type: Type[Exception]
    region = ...  # type: Any
    def __init__(self, **kwargs) -> None: ...
    def create_alias(self, alias_name: str, target_key_id: str) -> Optional[Dict[str, Any]]: ...
    def create_grant(self, key_id: str, grantee_principal: str, retiring_principal: Optional[str] = ..., operations: Optional[List[str]] = ..., constraints: Optional[Dict[str, Dict[str, str]]] = ..., grant_tokens: Optional[List[str]] = ...) -> Optional[Dict[str, Any]]: ...
    def create_key(self, policy: Optional[str] = ..., description: Optional[str] = ..., key_usage: Optional[str] = ...) -> Optional[Dict[str, Any]]: ...
    def decrypt(self, ciphertext_blob: bytes, encryption_context: Optional[Mapping[str, Any]] = ..., grant_tokens: Optional[List[str]] = ...) -> Optional[Dict[str, Any]]: ...
    def delete_alias(self, alias_name: str) -> Optional[Dict[str, Any]]: ...
    def describe_key(self, key_id: str) -> Optional[Dict[str, Any]]: ...
    def disable_key(self, key_id: str) -> Optional[Dict[str, Any]]: ...
    def disable_key_rotation(self, key_id: str) -> Optional[Dict[str, Any]]: ...
    def enable_key(self, key_id: str) -> Optional[Dict[str, Any]]: ...
    def enable_key_rotation(self, key_id: str) -> Optional[Dict[str, Any]]: ...
    def encrypt(self, key_id: str, plaintext: bytes, encryption_context: Optional[Mapping[str, Any]] = ..., grant_tokens: Optional[List[str]] = ...) -> Optional[Dict[str, Any]]: ...
    def generate_data_key(self, key_id: str, encryption_context: Optional[Mapping[str, Any]] = ..., number_of_bytes: Optional[int] = ..., key_spec: Optional[str] = ..., grant_tokens: Optional[List[str]] = ...) -> Optional[Dict[str, Any]]: ...
    def generate_data_key_without_plaintext(self, key_id: str, encryption_context: Optional[Mapping[str, Any]] = ..., key_spec: Optional[str] = ..., number_of_bytes: Optional[int] = ..., grant_tokens: Optional[List[str]] = ...) -> Optional[Dict[str, Any]]: ...
    def generate_random(self, number_of_bytes: Optional[int] = ...) -> Optional[Dict[str, Any]]: ...
    def get_key_policy(self, key_id: str, policy_name: str) -> Optional[Dict[str, Any]]: ...
    def get_key_rotation_status(self, key_id: str) -> Optional[Dict[str, Any]]: ...
    def list_aliases(self, limit: Optional[int] = ..., marker: Optional[str] = ...) -> Optional[Dict[str, Any]]: ...
    def list_grants(self, key_id: str, limit: Optional[int] = ..., marker: Optional[str] = ...) -> Optional[Dict[str, Any]]: ...
    def list_key_policies(self, key_id: str, limit: Optional[int] = ..., marker: Optional[str] = ...) -> Optional[Dict[str, Any]]: ...
    def list_keys(self, limit: Optional[int] = ..., marker: Optional[str] = ...) -> Optional[Dict[str, Any]]: ...
    def put_key_policy(self, key_id: str, policy_name: str, policy: str) -> Optional[Dict[str, Any]]: ...
    def re_encrypt(self, ciphertext_blob: bytes, destination_key_id: str, source_encryption_context: Optional[Mapping[str, Any]] = ..., destination_encryption_context: Optional[Mapping[str, Any]] = ..., grant_tokens: Optional[List[str]] = ...) -> Optional[Dict[str, Any]]: ...
    def retire_grant(self, grant_token: str) -> Optional[Dict[str, Any]]: ...
    def revoke_grant(self, key_id: str, grant_id: str) -> Optional[Dict[str, Any]]: ...
    def update_key_description(self, key_id: str, description: str) -> Optional[Dict[str, Any]]: ...



# Stubs for glob
# Based on http://docs.python.org/3/library/glob.html

from typing import List, Iterator, AnyStr
import sys

if sys.version_info >= (3, 6):
    def glob0(dirname: AnyStr, pattern: AnyStr) -> List[AnyStr]: ...
else:
    def glob0(dirname: AnyStr, basename: AnyStr) -> List[AnyStr]: ...

def glob1(dirname: AnyStr, pattern: AnyStr) -> List[AnyStr]: ...

if sys.version_info >= (3, 5):
    def glob(pathname: AnyStr, *, recursive: bool = ...) -> List[AnyStr]: ...
    def iglob(pathname: AnyStr, *, recursive: bool = ...) -> Iterator[AnyStr]: ...
else:
    def glob(pathname: AnyStr) -> List[AnyStr]: ...
    def iglob(pathname: AnyStr) -> Iterator[AnyStr]: ...

def escape(pathname: AnyStr) -> AnyStr: ...



# Stubs for email.header (Python 3.4)

from typing import Union, Optional, Any, List, Tuple
from email.charset import Charset

class Header:
    def __init__(self, s: Union[bytes, str, None] = ...,
                 charset: Union[Charset, str, None] = ...,
                 maxlinelen: Optional[int] = ...,
                 header_name: Optional[str] = ..., continuation_ws: str = ...,
                 errors: str = ...) -> None: ...
    def append(self, s: Union[bytes, str],
               charset: Union[Charset, str, None] = ...,
               errors: str = ...) -> None: ...
    def encode(self, splitchars: str = ..., maxlinelen: Optional[int] = ...,
               linesep: str = ...) -> str: ...
    def __str__(self) -> str: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...

def decode_header(header: Union[Header, str]) -> List[Tuple[bytes, Optional[str]]]: ...
def make_header(decoded_seq: List[Tuple[bytes, Optional[str]]],
                maxlinelen: Optional[int] =...,
                header_name: Optional[str] = ...,
                continuation_ws: str = ...) -> Header: ...



# Stubs for heapq

# Based on http://docs.python.org/3.2/library/heapq.html

import sys
from typing import TypeVar, List, Iterable, Any, Callable, Optional

_T = TypeVar('_T')

def heappush(heap: List[_T], item: _T) -> None: ...
def heappop(heap: List[_T]) -> _T: ...
def heappushpop(heap: List[_T], item: _T) -> _T: ...
def heapify(x: List[_T]) -> None: ...
def heapreplace(heap: List[_T], item: _T) -> _T: ...
if sys.version_info >= (3, 5):
    def merge(*iterables: Iterable[_T], key: Callable[[_T], Any] = ...,
              reverse: bool = ...) -> Iterable[_T]: ...
else:
    def merge(*iterables: Iterable[_T]) -> Iterable[_T]: ...
def nlargest(n: int, iterable: Iterable[_T],
             key: Optional[Callable[[_T], Any]] = ...) -> List[_T]: ...
def nsmallest(n: int, iterable: Iterable[_T],
              key: Callable[[_T], Any] = ...) -> List[_T]: ...



from typing import IO, List, Optional

from click.core import Context, Parameter


class ClickException(Exception):
    exit_code: int
    message: str

    def __init__(self, message: str) -> None:
        ...

    def format_message(self) -> str:
        ...

    def show(self, file=None) -> None:
        ...


class UsageError(ClickException):
    ctx: Optional[Context]

    def __init__(self, message: str, ctx: Optional[Context] = ...) -> None:
        ...

    def show(self, file: Optional[IO] = ...) -> None:
        ...


class BadParameter(UsageError):
    param: Optional[Parameter]
    param_hint: Optional[str]

    def __init__(
        self,
        message: str,
        ctx: Optional[Context] = ...,
        param: Optional[Parameter] = ...,
        param_hint: Optional[str] = ...
    ) -> None:
        ...


class MissingParameter(BadParameter):
    param_type: str  # valid values: 'parameter', 'option', 'argument'

    def __init__(
        self,
        message: Optional[str] = ...,
        ctx: Optional[Context] = ...,
        param: Optional[Parameter] = ...,
        param_hint: Optional[str] = ...,
        param_type: Optional[str] = ...
    ) -> None:
        ...


class NoSuchOption(UsageError):
    option_name: str
    possibilities: Optional[List[str]]

    def __init__(
        self,
        option_name: str,
        message: Optional[str] = ...,
        possibilities: Optional[List[str]] = ...,
        ctx: Optional[Context] = ...
    ) -> None:
        ...


class BadOptionUsage(UsageError):
    def __init__(self, message: str, ctx: Optional[Context] = ...) -> None:
        ...


class BadArgumentUsage(UsageError):
    def __init__(self, message: str, ctx: Optional[Context] = ...) -> None:
        ...


class FileError(ClickException):
    ui_filename: str
    filename: str

    def __init__(self, filename: str, hint: Optional[str] = ...) -> None:
        ...


class Abort(RuntimeError):
    ...



from typing import List, Union, Sequence, Optional, Dict


class Class:
    module = ...  # type: str
    name = ...  # type: str
    super = ...  # type: Optional[List[Union[Class, str]]]
    methods = ...  # type: Dict[str, int]
    file = ...  # type: int
    lineno = ...  # type: int

    def __init__(self,
                 module: str,
                 name: str,
                 super: Optional[List[Union[Class, str]]],
                 file: str,
                 lineno: int) -> None: ...


class Function:
    module = ...  # type: str
    name = ...  # type: str
    file = ...  # type: int
    lineno = ...  # type: int

    def __init__(self,
                 module: str,
                 name: str,
                 file: str,
                 lineno: int) -> None: ...


def readmodule(module: str,
               path: Optional[Sequence[str]] = ...
               ) -> Dict[str, Class]: ...


def readmodule_ex(module: str,
                  path: Optional[Sequence[str]] = ...
                  ) -> Dict[str, Union[Class, Function, List[str]]]: ...



from abc import abstractmethod
import asyncore
import socket
import sys
from typing import Optional, Sequence, Tuple, Union


class simple_producer:
    def __init__(self, data: bytes, buffer_size: int = ...) -> None: ...
    def more(self) -> bytes: ...

class async_chat(asyncore.dispatcher):
    ac_in_buffer_size = ...  # type: int
    ac_out_buffer_size = ...  # type: int
    def __init__(self, sock: Optional[socket.socket] = ..., map: Optional[asyncore._maptype] = ...) -> None: ...

    @abstractmethod
    def collect_incoming_data(self, data: bytes) -> None: ...
    @abstractmethod
    def found_terminator(self) -> None: ...
    def set_terminator(self, term: Union[bytes, int, None]) -> None: ...
    def get_terminator(self) -> Union[bytes, int, None]: ...
    def handle_read(self) -> None: ...
    def handle_write(self) -> None: ...
    def handle_close(self) -> None: ...
    def push(self, data: bytes) -> None: ...
    def push_with_producer(self, producer: simple_producer) -> None: ...
    def readable(self) -> bool: ...
    def writable(self) -> bool: ...
    def close_when_done(self) -> None: ...
    def initiate_send(self) -> None: ...
    def discard_buffers(self) -> None: ...

if sys.version_info < (3, 0):
    class fifo:
        def __init__(self, list: Sequence[Union[bytes, simple_producer]] = ...) -> None: ...
        def __len__(self) -> int: ...
        def is_empty(self) -> bool: ...
        def first(self) -> bytes: ...
        def push(self, data: Union[bytes, simple_producer]) -> None: ...
        def pop(self) -> Tuple[int, bytes]: ...



from typing import Tuple, Union, IO, Any, Optional, overload

AF_APPLETALK = ...  # type: int
AF_ASH = ...  # type: int
AF_ATMPVC = ...  # type: int
AF_ATMSVC = ...  # type: int
AF_AX25 = ...  # type: int
AF_BLUETOOTH = ...  # type: int
AF_BRIDGE = ...  # type: int
AF_DECnet = ...  # type: int
AF_ECONET = ...  # type: int
AF_INET = ...  # type: int
AF_INET6 = ...  # type: int
AF_IPX = ...  # type: int
AF_IRDA = ...  # type: int
AF_KEY = ...  # type: int
AF_LLC = ...  # type: int
AF_NETBEUI = ...  # type: int
AF_NETLINK = ...  # type: int
AF_NETROM = ...  # type: int
AF_PACKET = ...  # type: int
AF_PPPOX = ...  # type: int
AF_ROSE = ...  # type: int
AF_ROUTE = ...  # type: int
AF_SECURITY = ...  # type: int
AF_SNA = ...  # type: int
AF_TIPC = ...  # type: int
AF_UNIX = ...  # type: int
AF_UNSPEC = ...  # type: int
AF_WANPIPE = ...  # type: int
AF_X25 = ...  # type: int
AI_ADDRCONFIG = ...  # type: int
AI_ALL = ...  # type: int
AI_CANONNAME = ...  # type: int
AI_NUMERICHOST = ...  # type: int
AI_NUMERICSERV = ...  # type: int
AI_PASSIVE = ...  # type: int
AI_V4MAPPED = ...  # type: int
BDADDR_ANY = ...  # type: str
BDADDR_LOCAL = ...  # type: str
BTPROTO_HCI = ...  # type: int
BTPROTO_L2CAP = ...  # type: int
BTPROTO_RFCOMM = ...  # type: int
BTPROTO_SCO = ...  # type: int
EAI_ADDRFAMILY = ...  # type: int
EAI_AGAIN = ...  # type: int
EAI_BADFLAGS = ...  # type: int
EAI_FAIL = ...  # type: int
EAI_FAMILY = ...  # type: int
EAI_MEMORY = ...  # type: int
EAI_NODATA = ...  # type: int
EAI_NONAME = ...  # type: int
EAI_OVERFLOW = ...  # type: int
EAI_SERVICE = ...  # type: int
EAI_SOCKTYPE = ...  # type: int
EAI_SYSTEM = ...  # type: int
EBADF = ...  # type: int
EINTR = ...  # type: int
HCI_DATA_DIR = ...  # type: int
HCI_FILTER = ...  # type: int
HCI_TIME_STAMP = ...  # type: int
INADDR_ALLHOSTS_GROUP = ...  # type: int
INADDR_ANY = ...  # type: int
INADDR_BROADCAST = ...  # type: int
INADDR_LOOPBACK = ...  # type: int
INADDR_MAX_LOCAL_GROUP = ...  # type: int
INADDR_NONE = ...  # type: int
INADDR_UNSPEC_GROUP = ...  # type: int
IPPORT_RESERVED = ...  # type: int
IPPORT_USERRESERVED = ...  # type: int
IPPROTO_AH = ...  # type: int
IPPROTO_DSTOPTS = ...  # type: int
IPPROTO_EGP = ...  # type: int
IPPROTO_ESP = ...  # type: int
IPPROTO_FRAGMENT = ...  # type: int
IPPROTO_GRE = ...  # type: int
IPPROTO_HOPOPTS = ...  # type: int
IPPROTO_ICMP = ...  # type: int
IPPROTO_ICMPV6 = ...  # type: int
IPPROTO_IDP = ...  # type: int
IPPROTO_IGMP = ...  # type: int
IPPROTO_IP = ...  # type: int
IPPROTO_IPIP = ...  # type: int
IPPROTO_IPV6 = ...  # type: int
IPPROTO_NONE = ...  # type: int
IPPROTO_PIM = ...  # type: int
IPPROTO_PUP = ...  # type: int
IPPROTO_RAW = ...  # type: int
IPPROTO_ROUTING = ...  # type: int
IPPROTO_RSVP = ...  # type: int
IPPROTO_TCP = ...  # type: int
IPPROTO_TP = ...  # type: int
IPPROTO_UDP = ...  # type: int
IPV6_CHECKSUM = ...  # type: int
IPV6_DSTOPTS = ...  # type: int
IPV6_HOPLIMIT = ...  # type: int
IPV6_HOPOPTS = ...  # type: int
IPV6_JOIN_GROUP = ...  # type: int
IPV6_LEAVE_GROUP = ...  # type: int
IPV6_MULTICAST_HOPS = ...  # type: int
IPV6_MULTICAST_IF = ...  # type: int
IPV6_MULTICAST_LOOP = ...  # type: int
IPV6_NEXTHOP = ...  # type: int
IPV6_PKTINFO = ...  # type: int
IPV6_RECVDSTOPTS = ...  # type: int
IPV6_RECVHOPLIMIT = ...  # type: int
IPV6_RECVHOPOPTS = ...  # type: int
IPV6_RECVPKTINFO = ...  # type: int
IPV6_RECVRTHDR = ...  # type: int
IPV6_RECVTCLASS = ...  # type: int
IPV6_RTHDR = ...  # type: int
IPV6_RTHDRDSTOPTS = ...  # type: int
IPV6_RTHDR_TYPE_0 = ...  # type: int
IPV6_TCLASS = ...  # type: int
IPV6_UNICAST_HOPS = ...  # type: int
IPV6_V6ONLY = ...  # type: int
IP_ADD_MEMBERSHIP = ...  # type: int
IP_DEFAULT_MULTICAST_LOOP = ...  # type: int
IP_DEFAULT_MULTICAST_TTL = ...  # type: int
IP_DROP_MEMBERSHIP = ...  # type: int
IP_HDRINCL = ...  # type: int
IP_MAX_MEMBERSHIPS = ...  # type: int
IP_MULTICAST_IF = ...  # type: int
IP_MULTICAST_LOOP = ...  # type: int
IP_MULTICAST_TTL = ...  # type: int
IP_OPTIONS = ...  # type: int
IP_RECVOPTS = ...  # type: int
IP_RECVRETOPTS = ...  # type: int
IP_RETOPTS = ...  # type: int
IP_TOS = ...  # type: int
IP_TTL = ...  # type: int
MSG_CTRUNC = ...  # type: int
MSG_DONTROUTE = ...  # type: int
MSG_DONTWAIT = ...  # type: int
MSG_EOR = ...  # type: int
MSG_OOB = ...  # type: int
MSG_PEEK = ...  # type: int
MSG_TRUNC = ...  # type: int
MSG_WAITALL = ...  # type: int
MethodType = ...  # type: type
NETLINK_DNRTMSG = ...  # type: int
NETLINK_FIREWALL = ...  # type: int
NETLINK_IP6_FW = ...  # type: int
NETLINK_NFLOG = ...  # type: int
NETLINK_ROUTE = ...  # type: int
NETLINK_USERSOCK = ...  # type: int
NETLINK_XFRM = ...  # type: int
NI_DGRAM = ...  # type: int
NI_MAXHOST = ...  # type: int
NI_MAXSERV = ...  # type: int
NI_NAMEREQD = ...  # type: int
NI_NOFQDN = ...  # type: int
NI_NUMERICHOST = ...  # type: int
NI_NUMERICSERV = ...  # type: int
PACKET_BROADCAST = ...  # type: int
PACKET_FASTROUTE = ...  # type: int
PACKET_HOST = ...  # type: int
PACKET_LOOPBACK = ...  # type: int
PACKET_MULTICAST = ...  # type: int
PACKET_OTHERHOST = ...  # type: int
PACKET_OUTGOING = ...  # type: int
PF_PACKET = ...  # type: int
SHUT_RD = ...  # type: int
SHUT_RDWR = ...  # type: int
SHUT_WR = ...  # type: int
SOCK_DGRAM = ...  # type: int
SOCK_RAW = ...  # type: int
SOCK_RDM = ...  # type: int
SOCK_SEQPACKET = ...  # type: int
SOCK_STREAM = ...  # type: int
SOL_HCI = ...  # type: int
SOL_IP = ...  # type: int
SOL_SOCKET = ...  # type: int
SOL_TCP = ...  # type: int
SOL_TIPC = ...  # type: int
SOL_UDP = ...  # type: int
SOMAXCONN = ...  # type: int
SO_ACCEPTCONN = ...  # type: int
SO_BROADCAST = ...  # type: int
SO_DEBUG = ...  # type: int
SO_DONTROUTE = ...  # type: int
SO_ERROR = ...  # type: int
SO_KEEPALIVE = ...  # type: int
SO_LINGER = ...  # type: int
SO_OOBINLINE = ...  # type: int
SO_RCVBUF = ...  # type: int
SO_RCVLOWAT = ...  # type: int
SO_RCVTIMEO = ...  # type: int
SO_REUSEADDR = ...  # type: int
SO_REUSEPORT = ...  # type: int
SO_SNDBUF = ...  # type: int
SO_SNDLOWAT = ...  # type: int
SO_SNDTIMEO = ...  # type: int
SO_TYPE = ...  # type: int
SSL_ERROR_EOF = ...  # type: int
SSL_ERROR_INVALID_ERROR_CODE = ...  # type: int
SSL_ERROR_SSL = ...  # type: int
SSL_ERROR_SYSCALL = ...  # type: int
SSL_ERROR_WANT_CONNECT = ...  # type: int
SSL_ERROR_WANT_READ = ...  # type: int
SSL_ERROR_WANT_WRITE = ...  # type: int
SSL_ERROR_WANT_X509_LOOKUP = ...  # type: int
SSL_ERROR_ZERO_RETURN = ...  # type: int
TCP_CORK = ...  # type: int
TCP_DEFER_ACCEPT = ...  # type: int
TCP_INFO = ...  # type: int
TCP_KEEPCNT = ...  # type: int
TCP_KEEPIDLE = ...  # type: int
TCP_KEEPINTVL = ...  # type: int
TCP_LINGER2 = ...  # type: int
TCP_MAXSEG = ...  # type: int
TCP_NODELAY = ...  # type: int
TCP_QUICKACK = ...  # type: int
TCP_SYNCNT = ...  # type: int
TCP_WINDOW_CLAMP = ...  # type: int
TIPC_ADDR_ID = ...  # type: int
TIPC_ADDR_NAME = ...  # type: int
TIPC_ADDR_NAMESEQ = ...  # type: int
TIPC_CFG_SRV = ...  # type: int
TIPC_CLUSTER_SCOPE = ...  # type: int
TIPC_CONN_TIMEOUT = ...  # type: int
TIPC_CRITICAL_IMPORTANCE = ...  # type: int
TIPC_DEST_DROPPABLE = ...  # type: int
TIPC_HIGH_IMPORTANCE = ...  # type: int
TIPC_IMPORTANCE = ...  # type: int
TIPC_LOW_IMPORTANCE = ...  # type: int
TIPC_MEDIUM_IMPORTANCE = ...  # type: int
TIPC_NODE_SCOPE = ...  # type: int
TIPC_PUBLISHED = ...  # type: int
TIPC_SRC_DROPPABLE = ...  # type: int
TIPC_SUBSCR_TIMEOUT = ...  # type: int
TIPC_SUB_CANCEL = ...  # type: int
TIPC_SUB_PORTS = ...  # type: int
TIPC_SUB_SERVICE = ...  # type: int
TIPC_TOP_SRV = ...  # type: int
TIPC_WAIT_FOREVER = ...  # type: int
TIPC_WITHDRAWN = ...  # type: int
TIPC_ZONE_SCOPE = ...  # type: int

# PyCapsule
CAPI = ...  # type: Any

has_ipv6 = ...  # type: bool

class error(IOError): ...
class gaierror(error): ...
class timeout(error): ...

class SocketType(object):
    family = ...  # type: int
    type = ...  # type: int
    proto = ...  # type: int
    timeout = ...  # type: float

    def __init__(self, family: int = ..., type: int = ..., proto: int = ...) -> None: ...
    def accept(self) -> Tuple[SocketType, tuple]: ...
    def bind(self, address: tuple) -> None: ...
    def close(self) -> None: ...
    def connect(self, address: tuple) -> None:
        raise gaierror
        raise timeout
    def connect_ex(self, address: tuple) -> int: ...
    def dup(self) -> SocketType: ...
    def fileno(self) -> int: ...
    def getpeername(self) -> tuple: ...
    def getsockname(self) -> tuple: ...
    def getsockopt(self, level: int, option: int, buffersize: int = ...) -> str: ...
    def gettimeout(self) -> float: ...
    def listen(self, backlog: int) -> None:
        raise error
    def makefile(self, mode: str = ..., buffersize: int = ...) -> IO[Any]: ...
    def recv(self, buffersize: int, flags: int = ...) -> str: ...
    def recv_into(self, buffer: bytearray, nbytes: int = ..., flags: int = ...) -> int: ...
    def recvfrom(self, buffersize: int, flags: int = ...) -> tuple:
        raise error
    def recvfrom_into(self, buffer: bytearray, nbytes: int = ...,
                      flags: int = ...) -> int: ...
    def send(self, data: str, flags: int =...) -> int: ...
    def sendall(self, data: str, flags: int = ...) -> None: ...
    @overload
    def sendto(self, data: str, address: tuple) -> int: ...
    @overload
    def sendto(self, data: str, flags: int, address: tuple) -> int: ...
    def setblocking(self, flag: bool) -> None: ...
    def setsockopt(self, level: int, option: int, value: Union[int, str]) -> None: ...
    def settimeout(self, value: Optional[float]) -> None: ...
    def shutdown(self, flag: int) -> None: ...



from typing import Dict, Union
from urllib.response import addinfourl

# Stubs for urllib.error

class URLError(IOError):
    reason = ...  # type: Union[str, BaseException]
class HTTPError(URLError, addinfourl):
    code = ...  # type: int
    headers = ...  # type: Dict[str, str]
class ContentTooShortError(URLError): ...



from .bucketlistresultset import BucketListResultSet
from .connection import S3Connection
from .key import Key

from typing import Any, Dict, Optional, Text, Type, List

class S3WebsiteEndpointTranslate:
    trans_region = ...  # type: Dict[str, str]
    @classmethod
    def translate_region(self, reg: Text) -> str: ...

S3Permissions = ...  # type: List[str]

class Bucket:
    LoggingGroup = ...  # type: str
    BucketPaymentBody = ...  # type: str
    VersioningBody = ...  # type: str
    VersionRE = ...  # type: str
    MFADeleteRE = ...  # type: str
    name = ...  # type: Text
    connection = ...  # type: S3Connection
    key_class = ...  # type: Type[Key]
    def __init__(self, connection: Optional[S3Connection] = ..., name: Optional[Text] = ..., key_class: Type[Key] = ...) -> None: ...
    def __iter__(self): ...
    def __contains__(self, key_name) -> bool: ...
    def startElement(self, name, attrs, connection): ...
    creation_date = ...  # type: Any
    def endElement(self, name, value, connection): ...
    def set_key_class(self, key_class): ...
    def lookup(self, key_name, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_key(self, key_name, headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ..., response_headers: Optional[Dict[Text, Text]] = ..., validate: bool = ...) -> Key: ...
    def list(self, prefix: Text = ..., delimiter: Text = ..., marker: Text = ..., headers: Optional[Dict[Text, Text]] = ..., encoding_type: Optional[Any] = ...) -> BucketListResultSet: ...
    def list_versions(self, prefix: str = ..., delimiter: str = ..., key_marker: str = ..., version_id_marker: str = ..., headers: Optional[Dict[Text, Text]] = ..., encoding_type: Optional[Text] = ...) -> BucketListResultSet: ...
    def list_multipart_uploads(self, key_marker: str = ..., upload_id_marker: str = ..., headers: Optional[Dict[Text, Text]] = ..., encoding_type: Optional[Any] = ...): ...
    def validate_kwarg_names(self, kwargs, names): ...
    def get_all_keys(self, headers: Optional[Dict[Text, Text]] = ..., **params): ...
    def get_all_versions(self, headers: Optional[Dict[Text, Text]] = ..., **params): ...
    def validate_get_all_versions_params(self, params): ...
    def get_all_multipart_uploads(self, headers: Optional[Dict[Text, Text]] = ..., **params): ...
    def new_key(self, key_name: Optional[Any] = ...): ...
    def generate_url(self, expires_in, method: str = ..., headers: Optional[Dict[Text, Text]] = ..., force_http: bool = ..., response_headers: Optional[Dict[Text, Text]] = ..., expires_in_absolute: bool = ...): ...
    def delete_keys(self, keys, quiet: bool = ..., mfa_token: Optional[Any] = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def delete_key(self, key_name, headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ..., mfa_token: Optional[Any] = ...): ...
    def copy_key(self, new_key_name, src_bucket_name, src_key_name, metadata: Optional[Any] = ..., src_version_id: Optional[Any] = ..., storage_class: str = ..., preserve_acl: bool = ..., encrypt_key: bool = ..., headers: Optional[Dict[Text, Text]] = ..., query_args: Optional[Any] = ...): ...
    def set_canned_acl(self, acl_str, key_name: str = ..., headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ...): ...
    def get_xml_acl(self, key_name: str = ..., headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ...): ...
    def set_xml_acl(self, acl_str, key_name: str = ..., headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ..., query_args: str = ...): ...
    def set_acl(self, acl_or_str, key_name: str = ..., headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ...): ...
    def get_acl(self, key_name: str = ..., headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ...): ...
    def set_subresource(self, subresource, value, key_name: str = ..., headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ...): ...
    def get_subresource(self, subresource, key_name: str = ..., headers: Optional[Dict[Text, Text]] = ..., version_id: Optional[Any] = ...): ...
    def make_public(self, recursive: bool = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def add_email_grant(self, permission, email_address, recursive: bool = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def add_user_grant(self, permission, user_id, recursive: bool = ..., headers: Optional[Dict[Text, Text]] = ..., display_name: Optional[Any] = ...): ...
    def list_grants(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_location(self): ...
    def set_xml_logging(self, logging_str, headers: Optional[Dict[Text, Text]] = ...): ...
    def enable_logging(self, target_bucket, target_prefix: str = ..., grants: Optional[Any] = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def disable_logging(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_logging_status(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_as_logging_target(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_request_payment(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_request_payment(self, payer: str = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def configure_versioning(self, versioning, mfa_delete: bool = ..., mfa_token: Optional[Any] = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def get_versioning_status(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def configure_lifecycle(self, lifecycle_config, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_lifecycle_config(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def delete_lifecycle_configuration(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def configure_website(self, suffix: Optional[Any] = ..., error_key: Optional[Any] = ..., redirect_all_requests_to: Optional[Any] = ..., routing_rules: Optional[Any] = ..., headers: Optional[Dict[Text, Text]] = ...): ...
    def set_website_configuration(self, config, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_website_configuration_xml(self, xml, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_website_configuration(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_website_configuration_obj(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_website_configuration_with_xml(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_website_configuration_xml(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def delete_website_configuration(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_website_endpoint(self): ...
    def get_policy(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_policy(self, policy, headers: Optional[Dict[Text, Text]] = ...): ...
    def delete_policy(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_cors_xml(self, cors_xml, headers: Optional[Dict[Text, Text]] = ...): ...
    def set_cors(self, cors_config, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_cors_xml(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_cors(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def delete_cors(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def initiate_multipart_upload(self, key_name, headers: Optional[Dict[Text, Text]] = ..., reduced_redundancy: bool = ..., metadata: Optional[Any] = ..., encrypt_key: bool = ..., policy: Optional[Any] = ...): ...
    def complete_multipart_upload(self, key_name, upload_id, xml_body, headers: Optional[Dict[Text, Text]] = ...): ...
    def cancel_multipart_upload(self, key_name, upload_id, headers: Optional[Dict[Text, Text]] = ...): ...
    def delete(self, headers: Optional[Dict[Text, Text]] = ...): ...
    def get_tags(self): ...
    def get_xml_tags(self): ...
    def set_xml_tags(self, tag_str, headers: Optional[Dict[Text, Text]] = ..., query_args: str = ...): ...
    def set_tags(self, tags, headers: Optional[Dict[Text, Text]] = ...): ...
    def delete_tags(self, headers: Optional[Dict[Text, Text]] = ...): ...



"""Test cases for generating node-level dependencies (for fine-grained incremental checking)"""

import os
from typing import List, Tuple, Dict, Optional, Set
MYPY = False
if MYPY:
    from typing import DefaultDict
from collections import defaultdict

from mypy import build, defaults
from mypy.build import BuildSource
from mypy.errors import CompileError
from mypy.nodes import MypyFile, Expression
from mypy.options import Options
from mypy.server.deps import get_dependencies
from mypy.test.config import test_temp_dir
from mypy.test.data import DataDrivenTestCase, DataSuite
from mypy.test.helpers import assert_string_arrays_equal
from mypy.types import Type
from mypy.typestate import TypeState


# Only dependencies in these modules are dumped
dumped_modules = ['__main__', 'pkg', 'pkg.mod']


class GetDependenciesSuite(DataSuite):
    files = [
        'deps.test',
        'deps-types.test',
        'deps-generics.test',
        'deps-expressions.test',
        'deps-statements.test',
        'deps-classes.test',
    ]

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        src = '\n'.join(testcase.input)
        dump_all = '# __dump_all__' in src
        if testcase.name.endswith('python2'):
            python_version = defaults.PYTHON2_VERSION
        else:
            python_version = defaults.PYTHON3_VERSION
        messages, files, type_map = self.build(src, python_version)
        a = messages
        if files is None or type_map is None:
            if not a:
                a = ['Unknown compile error (likely syntax error in test case or fixture)']
        else:
            deps = defaultdict(set)  # type: DefaultDict[str, Set[str]]
            for module in files:
                if module in dumped_modules or dump_all and module not in ('abc',
                                                                           'typing',
                                                                           'mypy_extensions',
                                                                           'enum'):
                    new_deps = get_dependencies(files[module], type_map, python_version)
                    for source in new_deps:
                        deps[source].update(new_deps[source])

            TypeState.add_all_protocol_deps(deps)

            for source, targets in sorted(deps.items()):
                if source.startswith('<enum.'):
                    # Remove noise.
                    continue
                line = '%s -> %s' % (source, ', '.join(sorted(targets)))
                # Clean up output a bit
                line = line.replace('__main__', 'm')
                a.append(line)

        assert_string_arrays_equal(
            testcase.output, a,
            'Invalid output ({}, line {})'.format(testcase.file,
                                                  testcase.line))

    def build(self,
              source: str,
              python_version: Tuple[int, int]) -> Tuple[List[str],
                                                        Optional[Dict[str, MypyFile]],
                                                        Optional[Dict[Expression, Type]]]:
        options = Options()
        options.use_builtins_fixtures = True
        options.show_traceback = True
        options.cache_dir = os.devnull
        options.python_version = python_version
        options.export_types = True
        try:
            result = build.build(sources=[BuildSource('main', None, source)],
                                 options=options,
                                 alt_lib_path=test_temp_dir)
        except CompileError as e:
            # TODO: Should perhaps not return None here.
            return e.messages, None, None
        return result.errors, result.files, result.types



"""Test cases for the type checker: exporting inferred types"""

import re

from typing import Set

from mypy import build
from mypy.build import BuildSource
from mypy.test.config import test_temp_dir
from mypy.test.data import DataDrivenTestCase, DataSuite
from mypy.test.helpers import assert_string_arrays_equal
from mypy.util import short_type
from mypy.nodes import (
    NameExpr, TypeVarExpr, CallExpr, Expression, MypyFile, AssignmentStmt, IntExpr
)
from mypy.traverser import TraverserVisitor
from mypy.errors import CompileError
from mypy.options import Options


class TypeExportSuite(DataSuite):
    required_out_section = True
    files = ['typexport-basic.test']

    def run_case(self, testcase: DataDrivenTestCase) -> None:
        try:
            line = testcase.input[0]
            mask = ''
            if line.startswith('##'):
                mask = '(' + line[2:].strip() + ')$'

            src = '\n'.join(testcase.input)
            options = Options()
            options.strict_optional = False  # TODO: Enable strict optional checking
            options.use_builtins_fixtures = True
            options.show_traceback = True
            options.export_types = True
            result = build.build(sources=[BuildSource('main', None, src)],
                                 options=options,
                                 alt_lib_path=test_temp_dir)
            a = result.errors
            map = result.types
            nodes = map.keys()

            # Ignore NameExpr nodes of variables with explicit (trivial) types
            # to simplify output.
            searcher = SkippedNodeSearcher()
            for file in result.files.values():
                file.accept(searcher)
            ignored = searcher.nodes

            # Filter nodes that should be included in the output.
            keys = []
            for node in nodes:
                if node.line is not None and node.line != -1 and map[node]:
                    if ignore_node(node) or node in ignored:
                        continue
                    if (re.match(mask, short_type(node))
                            or (isinstance(node, NameExpr)
                                and re.match(mask, node.name))):
                        # Include node in output.
                        keys.append(node)

            for key in sorted(keys,
                              key=lambda n: (n.line, short_type(n),
                                             str(n) + str(map[n]))):
                ts = str(map[key]).replace('*', '')  # Remove erased tags
                ts = ts.replace('__main__.', '')
                a.append('{}({}) : {}'.format(short_type(key), key.line, ts))
        except CompileError as e:
            a = e.messages
        assert_string_arrays_equal(
            testcase.output, a,
            'Invalid type checker output ({}, line {})'.format(testcase.file,
                                                               testcase.line))


class SkippedNodeSearcher(TraverserVisitor):
    def __init__(self) -> None:
        self.nodes = set()  # type: Set[Expression]
        self.is_typing = False

    def visit_mypy_file(self, f: MypyFile) -> None:
        self.is_typing = f.fullname() == 'typing' or f.fullname() == 'builtins'
        super().visit_mypy_file(f)

    def visit_assignment_stmt(self, s: AssignmentStmt) -> None:
        if s.type or ignore_node(s.rvalue):
            for lvalue in s.lvalues:
                if isinstance(lvalue, NameExpr):
                    self.nodes.add(lvalue)
        super().visit_assignment_stmt(s)

    def visit_name_expr(self, n: NameExpr) -> None:
        self.skip_if_typing(n)

    def visit_int_expr(self, n: IntExpr) -> None:
        self.skip_if_typing(n)

    def skip_if_typing(self, n: Expression) -> None:
        if self.is_typing:
            self.nodes.add(n)


def ignore_node(node: Expression) -> bool:
    """Return True if node is to be omitted from test case output."""

    # We want to get rid of object() expressions in the typing module stub
    # and also TypeVar(...) expressions. Since detecting whether a node comes
    # from the typing module is not easy, we just to strip them all away.
    if isinstance(node, TypeVarExpr):
        return True
    if isinstance(node, NameExpr) and node.fullname == 'builtins.object':
        return True
    if isinstance(node, NameExpr) and node.fullname == 'builtins.None':
        return True
    if isinstance(node, CallExpr) and (ignore_node(node.callee) or
                                       node.analyzed):
        return True

    return False



from typing import Any
from tornado import httputil
from tornado.tcpserver import TCPServer
from tornado.util import Configurable

class HTTPServer(TCPServer, Configurable, httputil.HTTPServerConnectionDelegate):
    def __init__(self, *args, **kwargs) -> None: ...
    request_callback = ...  # type: Any
    no_keep_alive = ...  # type: Any
    xheaders = ...  # type: Any
    protocol = ...  # type: Any
    conn_params = ...  # type: Any
    def initialize(self, request_callback, no_keep_alive=..., io_loop=..., xheaders=..., ssl_options=..., protocol=..., decompress_request=..., chunk_size=..., max_header_size=..., idle_connection_timeout=..., body_timeout=..., max_body_size=..., max_buffer_size=...): ...
    @classmethod
    def configurable_base(cls): ...
    @classmethod
    def configurable_default(cls): ...
    def close_all_connections(self): ...
    def handle_stream(self, stream, address): ...
    def start_request(self, server_conn, request_conn): ...
    def on_close(self, server_conn): ...

class _HTTPRequestContext:
    address = ...  # type: Any
    protocol = ...  # type: Any
    address_family = ...  # type: Any
    remote_ip = ...  # type: Any
    def __init__(self, stream, address, protocol) -> None: ...

class _ServerRequestAdapter(httputil.HTTPMessageDelegate):
    server = ...  # type: Any
    connection = ...  # type: Any
    request = ...  # type: Any
    delegate = ...  # type: Any
    def __init__(self, server, server_conn, request_conn) -> None: ...
    def headers_received(self, start_line, headers): ...
    def data_received(self, chunk): ...
    def finish(self): ...
    def on_connection_close(self): ...

HTTPRequest = ...  # type: Any



from typing import Any, Callable, Iterator, Optional, Tuple

class JSONEncoder:
    item_separator = ...  # type: str
    key_separator = ...  # type: str

    skipkeys = ...  # type: bool
    ensure_ascii = ...  # type: bool
    check_circular = ...  # type: bool
    allow_nan = ...  # type: bool
    sort_keys = ...  # type: bool
    indent = ...  # type: int

    def __init__(self, skipkeys: bool = ..., ensure_ascii: bool = ...,
            check_circular: bool = ..., allow_nan: bool = ..., sort_keys: bool = ...,
            indent: Optional[int] = ..., separators: Optional[Tuple[str, str]] = ..., default: Optional[Callable] = ...) -> None: ...

    def default(self, o: Any) -> Any: ...
    def encode(self, o: Any) -> str: ...
    def iterencode(self, o: Any, _one_shot: bool = ...) -> Iterator[str]: ...



"""Self check mypy package"""

from mypy.test.helpers import Suite, run_mypy


class SelfCheckSuite(Suite):
    def test_mypy_package(self) -> None:
        run_mypy(['--config-file', 'mypy_self_check.ini', '-p', 'mypy'])

    def test_testrunner(self) -> None:
        run_mypy(['--config-file', 'mypy_self_check.ini',
                  '--no-warn-unused-configs', 'runtests.py', 'waiter.py'])



from typing import Any
from werkzeug.datastructures import CallbackDict

def generate_key(salt=None): ...

class ModificationTrackingDict(CallbackDict):
    modified = ...  # type: Any
    def __init__(self, *args, **kwargs): ...
    def copy(self): ...
    def __copy__(self): ...

class Session(ModificationTrackingDict):
    sid = ...  # type: Any
    new = ...  # type: Any
    def __init__(self, data, sid, new=False): ...
    @property
    def should_save(self): ...

class SessionStore:
    session_class = ...  # type: Any
    def __init__(self, session_class=None): ...
    def is_valid_key(self, key): ...
    def generate_key(self, salt=None): ...
    def new(self): ...
    def save(self, session): ...
    def save_if_modified(self, session): ...
    def delete(self, session): ...
    def get(self, sid): ...

class FilesystemSessionStore(SessionStore):
    path = ...  # type: Any
    filename_template = ...  # type: Any
    renew_missing = ...  # type: Any
    mode = ...  # type: Any
    def __init__(self, path=None, filename_template='', session_class=None, renew_missing=False, mode=420): ...
    def get_session_filename(self, sid): ...
    def save(self, session): ...
    def delete(self, session): ...
    def get(self, sid): ...
    def list(self): ...

class SessionMiddleware:
    app = ...  # type: Any
    store = ...  # type: Any
    cookie_name = ...  # type: Any
    cookie_age = ...  # type: Any
    cookie_expires = ...  # type: Any
    cookie_path = ...  # type: Any
    cookie_domain = ...  # type: Any
    cookie_secure = ...  # type: Any
    cookie_httponly = ...  # type: Any
    environ_key = ...  # type: Any
    def __init__(self, app, store, cookie_name='', cookie_age=None, cookie_expires=None, cookie_path='', cookie_domain=None, cookie_secure=None, cookie_httponly=False, environ_key=''): ...
    def __call__(self, environ, start_response): ...



import sys
from typing import List, Type

from asyncio.coroutines import (
    coroutine as coroutine,
    iscoroutinefunction as iscoroutinefunction,
    iscoroutine as iscoroutine,
)
from asyncio.protocols import (
    BaseProtocol as BaseProtocol,
    Protocol as Protocol,
    DatagramProtocol as DatagramProtocol,
    SubprocessProtocol as SubprocessProtocol,
)
from asyncio.streams import (
    StreamReader as StreamReader,
    StreamWriter as StreamWriter,
    StreamReaderProtocol as StreamReaderProtocol,
    open_connection as open_connection,
    start_server as start_server,
    IncompleteReadError as IncompleteReadError,
    LimitOverrunError as LimitOverrunError,
)
from asyncio.subprocess import (
    create_subprocess_exec as create_subprocess_exec,
    create_subprocess_shell as create_subprocess_shell,
)
from asyncio.transports import (
    BaseTransport as BaseTransport,
    ReadTransport as ReadTransport,
    WriteTransport as WriteTransport,
    Transport as Transport,
    DatagramTransport as DatagramTransport,
    SubprocessTransport as SubprocessTransport,
)
from asyncio.futures import (
    Future as Future,
    CancelledError as CancelledError,
    TimeoutError as TimeoutError,
    InvalidStateError as InvalidStateError,
    wrap_future as wrap_future,
)
from asyncio.tasks import (
    FIRST_COMPLETED as FIRST_COMPLETED,
    FIRST_EXCEPTION as FIRST_EXCEPTION,
    ALL_COMPLETED as ALL_COMPLETED,
    as_completed as as_completed,
    ensure_future as ensure_future,
    gather as gather,
    run_coroutine_threadsafe as run_coroutine_threadsafe,
    shield as shield,
    sleep as sleep,
    wait as wait,
    wait_for as wait_for,
    Task as Task,
)
from asyncio.events import (
    AbstractEventLoopPolicy as AbstractEventLoopPolicy,
    AbstractEventLoop as AbstractEventLoop,
    AbstractServer as AbstractServer,
    Handle as Handle,
    TimerHandle as TimerHandle,
    get_event_loop_policy as get_event_loop_policy,
    set_event_loop_policy as set_event_loop_policy,
    get_event_loop as get_event_loop,
    set_event_loop as set_event_loop,
    new_event_loop as new_event_loop,
    get_child_watcher as get_child_watcher,
    set_child_watcher as set_child_watcher,
)
from asyncio.queues import (
    Queue as Queue,
    PriorityQueue as PriorityQueue,
    LifoQueue as LifoQueue,
    QueueFull as QueueFull,
    QueueEmpty as QueueEmpty,
)
from asyncio.locks import (
    Lock as Lock,
    Event as Event,
    Condition as Condition,
    Semaphore as Semaphore,
    BoundedSemaphore as BoundedSemaphore,
)

if sys.version_info < (3, 5):
    from asyncio.queues import JoinableQueue as JoinableQueue
else:
    from asyncio.futures import isfuture as isfuture
    from asyncio.events import (
        _set_running_loop as _set_running_loop,
        _get_running_loop as _get_running_loop,
    )
if sys.platform != 'win32':
    from asyncio.streams import (
        open_unix_connection as open_unix_connection,
        start_unix_server as start_unix_server,
    )

# TODO: It should be possible to instantiate these classes, but mypy
# currently disallows this.
# See https://github.com/python/mypy/issues/1843
SelectorEventLoop = ...  # type: Type[AbstractEventLoop]
if sys.platform == 'win32':
    ProactorEventLoop = ...  # type: Type[AbstractEventLoop]
DefaultEventLoopPolicy = ...  # type: Type[AbstractEventLoopPolicy]

# TODO: AbstractChildWatcher (UNIX only)

__all__: List[str]



from contextlib import contextmanager
import os
import sys
import tempfile
from typing import Tuple, List, Generator, Optional
from unittest import TestCase, main

import mypy.api
from mypy.build import _get_site_packages_dirs, FileSystemCache
from mypy.test.config import package_path
from mypy.test.helpers import run_command
from mypy.util import try_find_python2_interpreter

SIMPLE_PROGRAM = """
from typedpkg.sample import ex
from typedpkg import dne
a = ex([''])
reveal_type(a)
"""


def check_mypy_run(cmd_line: List[str],
                   python_executable: str = sys.executable,
                   expected_out: str = '',
                   expected_err: str = '',
                   expected_returncode: int = 1,
                   venv_dir: Optional[str] = None) -> None:
    """Helper to run mypy and check the output."""
    if venv_dir is not None:
        old_dir = os.getcwd()
        os.chdir(venv_dir)
    try:
        if python_executable != sys.executable:
            cmd_line.append('--python-executable={}'.format(python_executable))
        out, err, returncode = mypy.api.run(cmd_line)
        assert out == expected_out, err
        assert err == expected_err, out
        assert returncode == expected_returncode, returncode
    finally:
        if venv_dir is not None:
            os.chdir(old_dir)


class TestPEP561(TestCase):

    @contextmanager
    def virtualenv(self,
                   python_executable: str = sys.executable
                   ) -> Generator[Tuple[str, str], None, None]:
        """Context manager that creates a virtualenv in a temporary directory

        returns the path to the created Python executable"""
        # Sadly, we need virtualenv, as the Python 3 venv module does not support creating a venv
        # for Python 2, and Python 2 does not have its own venv.
        with tempfile.TemporaryDirectory() as venv_dir:
            returncode, lines = run_command([sys.executable,
                                             '-m',
                                             'virtualenv',
                                             '-p{}'.format(python_executable),
                                            venv_dir], cwd=os.getcwd())
            if returncode != 0:
                err = '\n'.join(lines)
                self.fail("Failed to create venv. Do you have virtualenv installed?\n" + err)
            if sys.platform == 'win32':
                yield venv_dir, os.path.abspath(os.path.join(venv_dir, 'Scripts', 'python'))
            else:
                yield venv_dir, os.path.abspath(os.path.join(venv_dir, 'bin', 'python'))

    def install_package(self, pkg: str,
                        python_executable: str = sys.executable,
                        use_pip: bool = True,
                        editable: bool = False) -> None:
        """Context manager to temporarily install a package from test-data/packages/pkg/"""
        working_dir = os.path.join(package_path, pkg)
        if use_pip:
            install_cmd = [python_executable, '-m', 'pip', 'install']
            if editable:
                install_cmd.append('-e')
            install_cmd.append('.')
        else:
            install_cmd = [python_executable, 'setup.py']
            if editable:
                install_cmd.append('develop')
            else:
                install_cmd.append('install')
        returncode, lines = run_command(install_cmd, cwd=working_dir)
        if returncode != 0:
            self.fail('\n'.join(lines))

    def setUp(self) -> None:
        self.temp_file_dir = tempfile.TemporaryDirectory()
        self.tempfile = os.path.join(self.temp_file_dir.name, 'simple.py')
        with open(self.tempfile, 'w+') as file:
            file.write(SIMPLE_PROGRAM)
        self.msg_dne = \
            "{}:3: error: Module 'typedpkg' has no attribute 'dne'\n".format(self.tempfile)
        self.msg_list = \
            "{}:5: error: Revealed type is 'builtins.list[builtins.str]'\n".format(self.tempfile)
        self.msg_tuple = \
            "{}:5: error: Revealed type is 'builtins.tuple[builtins.str]'\n".format(self.tempfile)

    def tearDown(self) -> None:
        self.temp_file_dir.cleanup()

    def test_get_pkg_dirs(self) -> None:
        """Check that get_package_dirs works."""
        dirs = _get_site_packages_dirs(sys.executable, FileSystemCache())
        assert dirs

    def test_typedpkg_stub_package(self) -> None:
        with self.virtualenv() as venv:
            venv_dir, python_executable = venv
            self.install_package('typedpkg-stubs', python_executable)
            check_mypy_run(
                [self.tempfile],
                python_executable,
                expected_out=self.msg_dne + self.msg_list,
                venv_dir=venv_dir,
            )

    def test_typedpkg(self) -> None:
        with self.virtualenv() as venv:
            venv_dir, python_executable = venv
            self.install_package('typedpkg', python_executable)
            check_mypy_run(
                [self.tempfile],
                python_executable,
                expected_out=self.msg_tuple,
                venv_dir=venv_dir,
            )

    def test_stub_and_typed_pkg(self) -> None:
        with self.virtualenv() as venv:
            venv_dir, python_executable = venv
            self.install_package('typedpkg', python_executable)
            self.install_package('typedpkg-stubs', python_executable)
            check_mypy_run(
                [self.tempfile],
                python_executable,
                expected_out=self.msg_list,
                venv_dir=venv_dir,
            )

    def test_typedpkg_stubs_python2(self) -> None:
        python2 = try_find_python2_interpreter()
        if python2:
            with self.virtualenv(python2) as venv:
                venv_dir, py2 = venv
                self.install_package('typedpkg-stubs', py2)
                check_mypy_run(
                    [self.tempfile],
                    py2,
                    expected_out=self.msg_dne + self.msg_list,
                    venv_dir=venv_dir,
                )

    def test_typedpkg_python2(self) -> None:
        python2 = try_find_python2_interpreter()
        if python2:
            with self.virtualenv(python2) as venv:
                venv_dir, py2 = venv
                self.install_package('typedpkg', py2)
                check_mypy_run(
                    [self.tempfile],
                    py2,
                    expected_out=self.msg_tuple,
                    venv_dir=venv_dir,
                )

    def test_typedpkg_egg(self) -> None:
        with self.virtualenv() as venv:
            venv_dir, python_executable = venv
            self.install_package('typedpkg', python_executable, use_pip=False)
            check_mypy_run(
                [self.tempfile],
                python_executable,
                expected_out=self.msg_tuple,
                venv_dir=venv_dir,
            )

    def test_typedpkg_editable(self) -> None:
        with self.virtualenv() as venv:
            venv_dir, python_executable = venv
            self.install_package('typedpkg', python_executable, editable=True)
            check_mypy_run(
                [self.tempfile],
                python_executable,
                expected_out=self.msg_tuple,
                venv_dir=venv_dir,
            )


if __name__ == '__main__':
    main()



"""Ensure the argparse parser and Options class are in sync.

In particular, verify that the argparse defaults are the same as the Options
defaults, and that argparse doesn't assign any new members to the Options
object it creates.
"""
import argparse
import sys

import pytest  # type: ignore

from mypy.test.helpers import Suite, assert_equal
from mypy.options import Options
from mypy.main import (process_options, PythonExecutableInferenceError,
                       infer_python_version_and_executable)


class ArgSuite(Suite):
    def test_coherence(self) -> None:
        options = Options()
        _, parsed_options = process_options([], require_targets=False)
        # FIX: test this too. Requires changing working dir to avoid finding 'setup.cfg'
        options.config_file = parsed_options.config_file
        assert_equal(options, parsed_options)

    def test_executable_inference(self) -> None:
        """Test the --python-executable flag with --python-version"""
        sys_ver_str = '{ver.major}.{ver.minor}'.format(ver=sys.version_info)

        base = ['file.py']  # dummy file

        # test inference given one (infer the other)
        matching_version = base + ['--python-version={}'.format(sys_ver_str)]
        _, options = process_options(matching_version)
        assert options.python_version == sys.version_info[:2]
        assert options.python_executable == sys.executable

        matching_version = base + ['--python-executable={}'.format(sys.executable)]
        _, options = process_options(matching_version)
        assert options.python_version == sys.version_info[:2]
        assert options.python_executable == sys.executable

        # test inference given both
        matching_version = base + ['--python-version={}'.format(sys_ver_str),
                                   '--python-executable={}'.format(sys.executable)]
        _, options = process_options(matching_version)
        assert options.python_version == sys.version_info[:2]
        assert options.python_executable == sys.executable

        # test that we error if the version mismatch
        # argparse sys.exits on a parser.error, we need to check the raw inference function
        options = Options()

        special_opts = argparse.Namespace()
        special_opts.python_executable = sys.executable
        special_opts.python_version = (2, 10)  # obviously wrong
        special_opts.no_executable = None
        with pytest.raises(PythonExecutableInferenceError) as e:
            infer_python_version_and_executable(options, special_opts)
        assert str(e.value) == 'Python version (2, 10) did not match executable {}, got' \
                               ' version {}.'.format(sys.executable, sys.version_info[:2])

        # test that --no-site-packages will disable executable inference
        matching_version = base + ['--python-version={}'.format(sys_ver_str),
                                   '--no-site-packages']
        _, options = process_options(matching_version)
        assert options.python_version == sys.version_info[:2]
        assert options.python_executable is None



from typing import Any

string_types = ...  # type: Any
bytes_types = ...  # type: Any
LOST_STATES = ...  # type: Any
ENVI_VERSION = ...  # type: Any
ENVI_VERSION_KEY = ...  # type: Any
log = ...  # type: Any

class KazooClient:
    logger = ...  # type: Any
    handler = ...  # type: Any
    auth_data = ...  # type: Any
    default_acl = ...  # type: Any
    randomize_hosts = ...  # type: Any
    hosts = ...  # type: Any
    chroot = ...  # type: Any
    state = ...  # type: Any
    state_listeners = ...  # type: Any
    read_only = ...  # type: Any
    retry = ...  # type: Any
    Barrier = ...  # type: Any
    Counter = ...  # type: Any
    DoubleBarrier = ...  # type: Any
    ChildrenWatch = ...  # type: Any
    DataWatch = ...  # type: Any
    Election = ...  # type: Any
    NonBlockingLease = ...  # type: Any
    MultiNonBlockingLease = ...  # type: Any
    Lock = ...  # type: Any
    Party = ...  # type: Any
    Queue = ...  # type: Any
    LockingQueue = ...  # type: Any
    SetPartitioner = ...  # type: Any
    Semaphore = ...  # type: Any
    ShallowParty = ...  # type: Any
    def __init__(self, hosts=..., timeout=..., client_id=..., handler=..., default_acl=..., auth_data=..., read_only=..., randomize_hosts=..., connection_retry=..., command_retry=..., logger=..., **kwargs) -> None: ...
    @property
    def client_state(self): ...
    @property
    def client_id(self): ...
    @property
    def connected(self): ...
    def set_hosts(self, hosts, randomize_hosts=...): ...
    def add_listener(self, listener): ...
    def remove_listener(self, listener): ...
    def start(self, timeout=...): ...
    def start_async(self): ...
    def stop(self): ...
    def restart(self): ...
    def close(self): ...
    def command(self, cmd=...): ...
    def server_version(self, retries=...): ...
    def add_auth(self, scheme, credential): ...
    def add_auth_async(self, scheme, credential): ...
    def unchroot(self, path): ...
    def sync_async(self, path): ...
    def sync(self, path): ...
    def create(self, path, value=..., acl=..., ephemeral=..., sequence=..., makepath=...): ...
    def create_async(self, path, value=..., acl=..., ephemeral=..., sequence=..., makepath=...): ...
    def ensure_path(self, path, acl=...): ...
    def ensure_path_async(self, path, acl=...): ...
    def exists(self, path, watch=...): ...
    def exists_async(self, path, watch=...): ...
    def get(self, path, watch=...): ...
    def get_async(self, path, watch=...): ...
    def get_children(self, path, watch=..., include_data=...): ...
    def get_children_async(self, path, watch=..., include_data=...): ...
    def get_acls(self, path): ...
    def get_acls_async(self, path): ...
    def set_acls(self, path, acls, version=...): ...
    def set_acls_async(self, path, acls, version=...): ...
    def set(self, path, value, version=...): ...
    def set_async(self, path, value, version=...): ...
    def transaction(self): ...
    def delete(self, path, version=..., recursive=...): ...
    def delete_async(self, path, version=...): ...
    def reconfig(self, joining, leaving, new_members, from_config=...): ...
    def reconfig_async(self, joining, leaving, new_members, from_config): ...

class TransactionRequest:
    client = ...  # type: Any
    operations = ...  # type: Any
    committed = ...  # type: Any
    def __init__(self, client) -> None: ...
    def create(self, path, value=..., acl=..., ephemeral=..., sequence=...): ...
    def delete(self, path, version=...): ...
    def set_data(self, path, value, version=...): ...
    def check(self, path, version): ...
    def commit_async(self): ...
    def commit(self): ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, exc_tb): ...

class KazooState:
    ...



from typing import Any

class Cookie:
    version = ...  # type: Any
    name = ...  # type: Any
    value = ...  # type: Any
    port = ...  # type: Any
    port_specified = ...  # type: Any
    domain = ...  # type: Any
    domain_specified = ...  # type: Any
    domain_initial_dot = ...  # type: Any
    path = ...  # type: Any
    path_specified = ...  # type: Any
    secure = ...  # type: Any
    expires = ...  # type: Any
    discard = ...  # type: Any
    comment = ...  # type: Any
    comment_url = ...  # type: Any
    rfc2109 = ...  # type: Any
    def __init__(self, version, name, value, port, port_specified, domain, domain_specified, domain_initial_dot, path, path_specified, secure, expires, discard, comment, comment_url, rest, rfc2109=False): ...
    def has_nonstandard_attr(self, name): ...
    def get_nonstandard_attr(self, name, default=None): ...
    def set_nonstandard_attr(self, name, value): ...
    def is_expired(self, now=None): ...

class CookiePolicy:
    def set_ok(self, cookie, request): ...
    def return_ok(self, cookie, request): ...
    def domain_return_ok(self, domain, request): ...
    def path_return_ok(self, path, request): ...

class DefaultCookiePolicy(CookiePolicy):
    DomainStrictNoDots = ...  # type: Any
    DomainStrictNonDomain = ...  # type: Any
    DomainRFC2965Match = ...  # type: Any
    DomainLiberal = ...  # type: Any
    DomainStrict = ...  # type: Any
    netscape = ...  # type: Any
    rfc2965 = ...  # type: Any
    rfc2109_as_netscape = ...  # type: Any
    hide_cookie2 = ...  # type: Any
    strict_domain = ...  # type: Any
    strict_rfc2965_unverifiable = ...  # type: Any
    strict_ns_unverifiable = ...  # type: Any
    strict_ns_domain = ...  # type: Any
    strict_ns_set_initial_dollar = ...  # type: Any
    strict_ns_set_path = ...  # type: Any
    def __init__(self, blocked_domains=None, allowed_domains=None, netscape=True, rfc2965=False, rfc2109_as_netscape=None, hide_cookie2=False, strict_domain=False, strict_rfc2965_unverifiable=True, strict_ns_unverifiable=False, strict_ns_domain=..., strict_ns_set_initial_dollar=False, strict_ns_set_path=False): ...
    def blocked_domains(self): ...
    def set_blocked_domains(self, blocked_domains): ...
    def is_blocked(self, domain): ...
    def allowed_domains(self): ...
    def set_allowed_domains(self, allowed_domains): ...
    def is_not_allowed(self, domain): ...
    def set_ok(self, cookie, request): ...
    def set_ok_version(self, cookie, request): ...
    def set_ok_verifiability(self, cookie, request): ...
    def set_ok_name(self, cookie, request): ...
    def set_ok_path(self, cookie, request): ...
    def set_ok_domain(self, cookie, request): ...
    def set_ok_port(self, cookie, request): ...
    def return_ok(self, cookie, request): ...
    def return_ok_version(self, cookie, request): ...
    def return_ok_verifiability(self, cookie, request): ...
    def return_ok_secure(self, cookie, request): ...
    def return_ok_expires(self, cookie, request): ...
    def return_ok_port(self, cookie, request): ...
    def return_ok_domain(self, cookie, request): ...
    def domain_return_ok(self, domain, request): ...
    def path_return_ok(self, path, request): ...

class Absent: ...

class CookieJar:
    non_word_re = ...  # type: Any
    quote_re = ...  # type: Any
    strict_domain_re = ...  # type: Any
    domain_re = ...  # type: Any
    dots_re = ...  # type: Any
    magic_re = ...  # type: Any
    def __init__(self, policy=None): ...
    def set_policy(self, policy): ...
    def add_cookie_header(self, request): ...
    def make_cookies(self, response, request): ...
    def set_cookie_if_ok(self, cookie, request): ...
    def set_cookie(self, cookie): ...
    def extract_cookies(self, response, request): ...
    def clear(self, domain=None, path=None, name=None): ...
    def clear_session_cookies(self): ...
    def clear_expired_cookies(self): ...
    def __iter__(self): ...
    def __len__(self): ...

class LoadError(IOError): ...

class FileCookieJar(CookieJar):
    filename = ...  # type: Any
    delayload = ...  # type: Any
    def __init__(self, filename=None, delayload=False, policy=None): ...
    def save(self, filename=None, ignore_discard=False, ignore_expires=False): ...
    def load(self, filename=None, ignore_discard=False, ignore_expires=False): ...
    def revert(self, filename=None, ignore_discard=False, ignore_expires=False): ...

MozillaCookieJar = FileCookieJar
LWPCookieJar = FileCookieJar
def lwp_cookie_str(cookie: Cookie) -> str: ...



# Stubs for mock

import sys
from typing import Any, Optional, Text, Type

FILTER_DIR = ...  # type: Any

class _slotted: ...

class _SentinelObject:
    name = ...  # type: Any
    def __init__(self, name: Any) -> None: ...

class _Sentinel:
    def __init__(self) -> None: ...
    def __getattr__(self, name: str) -> Any: ...

sentinel = ...  # type: Any
DEFAULT = ...  # type: Any

class _CallList(list):
    def __contains__(self, value: Any) -> bool: ...

class _MockIter:
    obj = ...  # type: Any
    def __init__(self, obj: Any) -> None: ...
    def __iter__(self) -> Any: ...
    def __next__(self) -> Any: ...

class Base:
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

# TODO: Defining this and other mock classes as classes in this stub causes
#       many false positives with mypy and production code. See if we can
#       improve mypy somehow and use a class with an "Any" base class.
NonCallableMock: Any

class CallableMixin(Base):
    side_effect = ...  # type: Any
    def __init__(self, spec: Optional[Any] = ..., side_effect: Optional[Any] = ..., return_value: Any = ..., wraps: Optional[Any] = ..., name: Optional[Any] = ..., spec_set: Optional[Any] = ..., parent: Optional[Any] = ..., _spec_state: Optional[Any] = ..., _new_name: Any = ..., _new_parent: Optional[Any] = ..., **kwargs: Any) -> None: ...
    def __call__(_mock_self, *args: Any, **kwargs: Any) -> Any: ...

Mock: Any

class _patch:
    attribute_name = ...  # type: Any
    getter = ...  # type: Any
    attribute = ...  # type: Any
    new = ...  # type: Any
    new_callable = ...  # type: Any
    spec = ...  # type: Any
    create = ...  # type: bool
    has_local = ...  # type: Any
    spec_set = ...  # type: Any
    autospec = ...  # type: Any
    kwargs = ...  # type: Any
    additional_patchers = ...  # type: Any
    def __init__(self, getter: Any, attribute: Any, new: Any, spec: Any, create: Any, spec_set: Any, autospec: Any, new_callable: Any, kwargs: Any) -> None: ...
    def copy(self) -> Any: ...
    def __call__(self, func: Any) -> Any: ...
    def decorate_class(self, klass: Any) -> Any: ...
    def decorate_callable(self, func: Any) -> Any: ...
    def get_original(self) -> Any: ...
    target = ...  # type: Any
    temp_original = ...  # type: Any
    is_local = ...  # type: Any
    def __enter__(self) -> Any: ...
    def __exit__(self, *exc_info: Any) -> Any: ...
    def start(self) -> Any: ...
    def stop(self) -> Any: ...

class _patch_dict:
    in_dict = ...  # type: Any
    values = ...  # type: Any
    clear = ...  # type: Any
    def __init__(self, in_dict: Any, values: Any = ..., clear: Any = ..., **kwargs: Any) -> None: ...
    def __call__(self, f: Any) -> Any: ...
    def decorate_class(self, klass: Any) -> Any: ...
    def __enter__(self) -> Any: ...
    def __exit__(self, *args: Any) -> Any: ...
    start = ...  # type: Any
    stop = ...  # type: Any

class _patcher:
    TEST_PREFIX = ...  # type: str
    dict = ...  # type: Type[_patch_dict]
    def __call__(self, target: Any, new: Optional[Any] = ..., spec: Optional[Any] = ..., create: bool = ..., spec_set: Optional[Any] = ..., autospec: Optional[Any] = ..., new_callable: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def object(self, target: Any, attribute: Text, new: Optional[Any] = ..., spec: Optional[Any] = ..., create: bool = ..., spec_set: Optional[Any] = ..., autospec: Optional[Any] = ..., new_callable: Optional[Any] = ..., **kwargs: Any) -> _patch: ...
    def multiple(self, target: Any, spec: Optional[Any] = ..., create: bool = ..., spec_set: Optional[Any] = ..., autospec: Optional[Any] = ..., new_callable: Optional[Any] = ..., **kwargs: Any) -> Any: ...
    def stopall(self) -> None: ...

patch = ...  # type: _patcher

class MagicMixin:
    def __init__(self, *args: Any, **kw: Any) -> None: ...

NonCallableMagicMock: Any
MagicMock: Any

class MagicProxy:
    name = ...  # type: Any
    parent = ...  # type: Any
    def __init__(self, name: Any, parent: Any) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def create_mock(self) -> Any: ...
    def __get__(self, obj: Any, _type: Optional[Any] = ...) -> Any: ...

class _ANY:
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...

ANY = ...  # type: Any

class _Call(tuple):
    def __new__(cls, value: Any = ..., name: Optional[Any] = ..., parent: Optional[Any] = ..., two: bool = ..., from_kall: bool = ...) -> Any: ...
    name = ...  # type: Any
    parent = ...  # type: Any
    from_kall = ...  # type: Any
    def __init__(self, value: Any = ..., name: Optional[Any] = ..., parent: Optional[Any] = ..., two: bool = ..., from_kall: bool = ...) -> None: ...
    def __eq__(self, other: Any) -> bool: ...
    __ne__ = ...  # type: Any
    def __call__(self, *args: Any, **kwargs: Any) -> Any: ...
    def __getattr__(self, attr: Any) -> Any: ...
    def count(self, *args: Any, **kwargs: Any) -> Any: ...
    def index(self, *args: Any, **kwargs: Any) -> Any: ...
    def call_list(self) -> Any: ...

call = ...  # type: Any

def create_autospec(spec: Any, spec_set: Any = ..., instance: Any = ..., _parent: Optional[Any] = ..., _name: Optional[Any] = ..., **kwargs: Any) -> Any: ...

class _SpecState:
    spec = ...  # type: Any
    ids = ...  # type: Any
    spec_set = ...  # type: Any
    parent = ...  # type: Any
    instance = ...  # type: Any
    name = ...  # type: Any
    def __init__(self, spec: Any, spec_set: Any = ..., parent: Optional[Any] = ..., name: Optional[Any] = ..., ids: Optional[Any] = ..., instance: Any = ...) -> None: ...

def mock_open(mock: Optional[Any] = ..., read_data: Any = ...) -> Any: ...

PropertyMock: Any

if sys.version_info >= (3, 7):
    def seal(mock: Any) -> None: ...



from typing import ContextManager, Iterator, Generic, TypeVar

_T = TypeVar("_T")

class ProgressBar(object, Generic[_T]):
    def update(self, n_steps: int) -> None: ...
    def finish(self) -> None: ...
    def __enter__(self) -> ProgressBar[_T]: ...
    def __exit__(self, exc_type, exc_value, tb) -> None: ...
    def __iter__(self) -> ProgressBar[_T]: ...
    def next(self) -> _T: ...
    def __next__(self) -> _T: ...



import sys
from typing import Any

if sys.version_info < (3,):
    from SocketServer import ThreadingMixIn, ForkingMixIn
    from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
else:
    from socketserver import ThreadingMixIn, ForkingMixIn
    from http.server import HTTPServer, BaseHTTPRequestHandler

class _SslDummy:
    def __getattr__(self, name): ...

ssl = ...  # type: Any
LISTEN_QUEUE = ...  # type: Any
can_open_by_fd = ...  # type: Any

class WSGIRequestHandler(BaseHTTPRequestHandler):
    @property
    def server_version(self): ...
    def make_environ(self): ...
    environ = ...  # type: Any
    close_connection = ...  # type: Any
    def run_wsgi(self): ...
    def handle(self): ...
    def initiate_shutdown(self): ...
    def connection_dropped(self, error, environ=None): ...
    raw_requestline = ...  # type: Any
    def handle_one_request(self): ...
    def send_response(self, code, message=None): ...
    def version_string(self): ...
    def address_string(self): ...
    def port_integer(self): ...
    def log_request(self, code='', size=''): ...
    def log_error(self, *args): ...
    def log_message(self, format, *args): ...
    def log(self, type, message, *args): ...

BaseRequestHandler = ...  # type: Any

def generate_adhoc_ssl_pair(cn=None): ...
def make_ssl_devcert(base_path, host=None, cn=None): ...
def generate_adhoc_ssl_context(): ...
def load_ssl_context(cert_file, pkey_file=None, protocol=None): ...

class _SSLContext:
    def __init__(self, protocol): ...
    def load_cert_chain(self, certfile, keyfile=None, password=None): ...
    def wrap_socket(self, sock, **kwargs): ...

def is_ssl_error(error=None): ...
def select_ip_version(host, port): ...

class BaseWSGIServer(HTTPServer):
    multithread = ...  # type: Any
    multiprocess = ...  # type: Any
    request_queue_size = ...  # type: Any
    address_family = ...  # type: Any
    app = ...  # type: Any
    passthrough_errors = ...  # type: Any
    shutdown_signal = ...  # type: Any
    host = ...  # type: Any
    port = ...  # type: Any
    socket = ...  # type: Any
    server_address = ...  # type: Any
    ssl_context = ...  # type: Any
    def __init__(self, host, port, app, handler=None, passthrough_errors=False, ssl_context=None, fd=None): ...
    def log(self, type, message, *args): ...
    def serve_forever(self): ...
    def handle_error(self, request, client_address): ...
    def get_request(self): ...

class ThreadedWSGIServer(ThreadingMixIn, BaseWSGIServer):
    multithread = ...  # type: Any
    daemon_threads = ...  # type: Any

class ForkingWSGIServer(ForkingMixIn, BaseWSGIServer):
    multiprocess = ...  # type: Any
    max_children = ...  # type: Any
    def __init__(self, host, port, app, processes=40, handler=None, passthrough_errors=False, ssl_context=None, fd=None): ...

def make_server(host=None, port=None, app=None, threaded=False, processes=1, request_handler=None, passthrough_errors=False, ssl_context=None, fd=None): ...
def is_running_from_reloader(): ...
def run_simple(hostname, port, application, use_reloader=False, use_debugger=False, use_evalex=True, extra_files=None, reloader_interval=1, reloader_type='', threaded=False, processes=1, request_handler=None, static_files=None, passthrough_errors=False, ssl_context=None): ...
def run_with_reloader(*args, **kwargs): ...
def main(): ...



# Stubs for socket
# Ron Murawski <ron@horizonchess.com>

# based on: http://docs.python.org/3.2/library/socket.html
# see: http://hg.python.org/cpython/file/3d0686d90f55/Lib/socket.py
# see: http://nullege.com/codes/search/socket
# adapted for Python 2.7 by Michal Pokorny
import sys
from typing import Any, Iterable, Tuple, List, Optional, Union, overload, TypeVar

_WriteBuffer = Union[bytearray, memoryview]

# ----- variables and constants -----

AF_UNIX: AddressFamily
AF_INET: AddressFamily
AF_INET6: AddressFamily
SOCK_STREAM: SocketKind
SOCK_DGRAM: SocketKind
SOCK_RAW: SocketKind
SOCK_RDM: SocketKind
SOCK_SEQPACKET: SocketKind
SOCK_CLOEXEC: SocketKind
SOCK_NONBLOCK: SocketKind
SOMAXCONN: int
has_ipv6: bool
_GLOBAL_DEFAULT_TIMEOUT: Any
SocketType: Any
SocketIO: Any

# These are flags that may exist on Python 3.6. Many don't exist on all platforms.
AF_AAL5: AddressFamily
AF_APPLETALK: AddressFamily
AF_ASH: AddressFamily
AF_ATMPVC: AddressFamily
AF_ATMSVC: AddressFamily
AF_AX25: AddressFamily
AF_BLUETOOTH: AddressFamily
AF_BRIDGE: AddressFamily
AF_CAN: AddressFamily
AF_DECnet: AddressFamily
AF_ECONET: AddressFamily
AF_IPX: AddressFamily
AF_IRDA: AddressFamily
AF_KEY: AddressFamily
AF_LLC: AddressFamily
AF_NETBEUI: AddressFamily
AF_NETLINK: AddressFamily
AF_NETROM: AddressFamily
AF_PACKET: AddressFamily
AF_PPPOX: AddressFamily
AF_RDS: AddressFamily
AF_ROSE: AddressFamily
AF_ROUTE: AddressFamily
AF_SECURITY: AddressFamily
AF_SNA: AddressFamily
AF_SYSTEM: AddressFamily
AF_TIPC: AddressFamily
AF_UNSPEC: AddressFamily
AF_WANPIPE: AddressFamily
AF_X25: AddressFamily
AI_ADDRCONFIG: AddressInfo
AI_ALL: AddressInfo
AI_CANONNAME: AddressInfo
AI_DEFAULT: AddressInfo
AI_MASK: AddressInfo
AI_NUMERICHOST: AddressInfo
AI_NUMERICSERV: AddressInfo
AI_PASSIVE: AddressInfo
AI_V4MAPPED: AddressInfo
AI_V4MAPPED_CFG: AddressInfo
BDADDR_ANY: str
BDADDR_LOCAL: str
BTPROTO_HCI: int
BTPROTO_L2CAP: int
BTPROTO_RFCOMM: int
BTPROTO_SCO: int
CAN_EFF_FLAG: int
CAN_EFF_MASK: int
CAN_ERR_FLAG: int
CAN_ERR_MASK: int
CAN_RAW: int
CAN_RAW_ERR_FILTER: int
CAN_RAW_FILTER: int
CAN_RAW_LOOPBACK: int
CAN_RAW_RECV_OWN_MSGS: int
CAN_RTR_FLAG: int
CAN_SFF_MASK: int
CAPI: int
EAGAIN: int
EAI_ADDRFAMILY: int
EAI_AGAIN: int
EAI_BADFLAGS: int
EAI_BADHINTS: int
EAI_FAIL: int
EAI_FAMILY: int
EAI_MAX: int
EAI_MEMORY: int
EAI_NODATA: int
EAI_NONAME: int
EAI_OVERFLOW: int
EAI_PROTOCOL: int
EAI_SERVICE: int
EAI_SOCKTYPE: int
EAI_SYSTEM: int
EBADF: int
EINTR: int
EWOULDBLOCK: int
HCI_DATA_DIR: int
HCI_FILTER: int
HCI_TIME_STAMP: int
INADDR_ALLHOSTS_GROUP: int
INADDR_ANY: int
INADDR_BROADCAST: int
INADDR_LOOPBACK: int
INADDR_MAX_LOCAL_GROUP: int
INADDR_NONE: int
INADDR_UNSPEC_GROUP: int
IPPORT_RESERVED: int
IPPORT_USERRESERVED: int
IPPROTO_AH: int
IPPROTO_BIP: int
IPPROTO_DSTOPTS: int
IPPROTO_EGP: int
IPPROTO_EON: int
IPPROTO_ESP: int
IPPROTO_FRAGMENT: int
IPPROTO_GGP: int
IPPROTO_GRE: int
IPPROTO_HELLO: int
IPPROTO_HOPOPTS: int
IPPROTO_ICMP: int
IPPROTO_ICMPV6: int
IPPROTO_IDP: int
IPPROTO_IGMP: int
IPPROTO_IP: int
IPPROTO_IPCOMP: int
IPPROTO_IPIP: int
IPPROTO_IPV4: int
IPPROTO_IPV6: int
IPPROTO_MAX: int
IPPROTO_MOBILE: int
IPPROTO_ND: int
IPPROTO_NONE: int
IPPROTO_PIM: int
IPPROTO_PUP: int
IPPROTO_RAW: int
IPPROTO_ROUTING: int
IPPROTO_RSVP: int
IPPROTO_SCTP: int
IPPROTO_TCP: int
IPPROTO_TP: int
IPPROTO_UDP: int
IPPROTO_VRRP: int
IPPROTO_XTP: int
IPV6_CHECKSUM: int
IPV6_DONTFRAG: int
IPV6_DSTOPTS: int
IPV6_HOPLIMIT: int
IPV6_HOPOPTS: int
IPV6_JOIN_GROUP: int
IPV6_LEAVE_GROUP: int
IPV6_MULTICAST_HOPS: int
IPV6_MULTICAST_IF: int
IPV6_MULTICAST_LOOP: int
IPV6_NEXTHOP: int
IPV6_PATHMTU: int
IPV6_PKTINFO: int
IPV6_RECVDSTOPTS: int
IPV6_RECVHOPLIMIT: int
IPV6_RECVHOPOPTS: int
IPV6_RECVPATHMTU: int
IPV6_RECVPKTINFO: int
IPV6_RECVRTHDR: int
IPV6_RECVTCLASS: int
IPV6_RTHDR: int
IPV6_RTHDR_TYPE_0: int
IPV6_RTHDRDSTOPTS: int
IPV6_TCLASS: int
IPV6_UNICAST_HOPS: int
IPV6_USE_MIN_MTU: int
IPV6_V6ONLY: int
IP_ADD_MEMBERSHIP: int
IP_DEFAULT_MULTICAST_LOOP: int
IP_DEFAULT_MULTICAST_TTL: int
IP_DROP_MEMBERSHIP: int
IP_HDRINCL: int
IP_MAX_MEMBERSHIPS: int
IP_MULTICAST_IF: int
IP_MULTICAST_LOOP: int
IP_MULTICAST_TTL: int
IP_OPTIONS: int
IP_RECVDSTADDR: int
IP_RECVOPTS: int
IP_RECVRETOPTS: int
IP_RETOPTS: int
IP_TOS: int
IP_TRANSPARENT: int
IP_TTL: int
IPX_TYPE: int
LOCAL_PEERCRED: int
MSG_BCAST: MsgFlag
MSG_BTAG: MsgFlag
MSG_CMSG_CLOEXEC: MsgFlag
MSG_CONFIRM: MsgFlag
MSG_CTRUNC: MsgFlag
MSG_DONTROUTE: MsgFlag
MSG_DONTWAIT: MsgFlag
MSG_EOF: MsgFlag
MSG_EOR: MsgFlag
MSG_ERRQUEUE: MsgFlag
MSG_ETAG: MsgFlag
MSG_FASTOPEN: MsgFlag
MSG_MCAST: MsgFlag
MSG_MORE: MsgFlag
MSG_NOSIGNAL: MsgFlag
MSG_NOTIFICATION: MsgFlag
MSG_OOB: MsgFlag
MSG_PEEK: MsgFlag
MSG_TRUNC: MsgFlag
MSG_WAITALL: MsgFlag
NETLINK_ARPD: int
NETLINK_CRYPTO: int
NETLINK_DNRTMSG: int
NETLINK_FIREWALL: int
NETLINK_IP6_FW: int
NETLINK_NFLOG: int
NETLINK_ROUTE6: int
NETLINK_ROUTE: int
NETLINK_SKIP: int
NETLINK_TAPBASE: int
NETLINK_TCPDIAG: int
NETLINK_USERSOCK: int
NETLINK_W1: int
NETLINK_XFRM: int
NI_DGRAM: int
NI_MAXHOST: int
NI_MAXSERV: int
NI_NAMEREQD: int
NI_NOFQDN: int
NI_NUMERICHOST: int
NI_NUMERICSERV: int
PACKET_BROADCAST: int
PACKET_FASTROUTE: int
PACKET_HOST: int
PACKET_LOOPBACK: int
PACKET_MULTICAST: int
PACKET_OTHERHOST: int
PACKET_OUTGOING: int
PF_CAN: int
PF_PACKET: int
PF_RDS: int
PF_SYSTEM: int
SCM_CREDENTIALS: int
SCM_CREDS: int
SCM_RIGHTS: int
SHUT_RD: int
SHUT_RDWR: int
SHUT_WR: int
SOL_ATALK: int
SOL_AX25: int
SOL_CAN_BASE: int
SOL_CAN_RAW: int
SOL_HCI: int
SOL_IP: int
SOL_IPX: int
SOL_NETROM: int
SOL_RDS: int
SOL_ROSE: int
SOL_SOCKET: int
SOL_TCP: int
SOL_TIPC: int
SOL_UDP: int
SO_ACCEPTCONN: int
SO_BINDTODEVICE: int
SO_BROADCAST: int
SO_DEBUG: int
SO_DONTROUTE: int
SO_ERROR: int
SO_EXCLUSIVEADDRUSE: int
SO_KEEPALIVE: int
SO_LINGER: int
SO_MARK: int
SO_OOBINLINE: int
SO_PASSCRED: int
SO_PEERCRED: int
SO_PRIORITY: int
SO_RCVBUF: int
SO_RCVLOWAT: int
SO_RCVTIMEO: int
SO_REUSEADDR: int
SO_REUSEPORT: int
SO_SETFIB: int
SO_SNDBUF: int
SO_SNDLOWAT: int
SO_SNDTIMEO: int
SO_TYPE: int
SO_USELOOPBACK: int
SYSPROTO_CONTROL: int
TCP_CORK: int
TCP_DEFER_ACCEPT: int
TCP_FASTOPEN: int
TCP_INFO: int
TCP_KEEPCNT: int
TCP_KEEPIDLE: int
TCP_KEEPINTVL: int
TCP_LINGER2: int
TCP_MAXSEG: int
TCP_NODELAY: int
TCP_NOTSENT_LOWAT: int
TCP_QUICKACK: int
TCP_SYNCNT: int
TCP_WINDOW_CLAMP: int
TIPC_ADDR_ID: int
TIPC_ADDR_NAME: int
TIPC_ADDR_NAMESEQ: int
TIPC_CFG_SRV: int
TIPC_CLUSTER_SCOPE: int
TIPC_CONN_TIMEOUT: int
TIPC_CRITICAL_IMPORTANCE: int
TIPC_DEST_DROPPABLE: int
TIPC_HIGH_IMPORTANCE: int
TIPC_IMPORTANCE: int
TIPC_LOW_IMPORTANCE: int
TIPC_MEDIUM_IMPORTANCE: int
TIPC_NODE_SCOPE: int
TIPC_PUBLISHED: int
TIPC_SRC_DROPPABLE: int
TIPC_SUB_CANCEL: int
TIPC_SUB_PORTS: int
TIPC_SUB_SERVICE: int
TIPC_SUBSCR_TIMEOUT: int
TIPC_TOP_SRV: int
TIPC_WAIT_FOREVER: int
TIPC_WITHDRAWN: int
TIPC_ZONE_SCOPE: int

if sys.version_info >= (3, 3):
    RDS_CANCEL_SENT_TO: int
    RDS_CMSG_RDMA_ARGS: int
    RDS_CMSG_RDMA_DEST: int
    RDS_CMSG_RDMA_MAP: int
    RDS_CMSG_RDMA_STATUS: int
    RDS_CMSG_RDMA_UPDATE: int
    RDS_CONG_MONITOR: int
    RDS_FREE_MR: int
    RDS_GET_MR: int
    RDS_GET_MR_FOR_DEST: int
    RDS_RDMA_DONTWAIT: int
    RDS_RDMA_FENCE: int
    RDS_RDMA_INVALIDATE: int
    RDS_RDMA_NOTIFY_ME: int
    RDS_RDMA_READWRITE: int
    RDS_RDMA_SILENT: int
    RDS_RDMA_USE_ONCE: int
    RDS_RECVERR: int

if sys.version_info >= (3, 4):
    CAN_BCM: int
    CAN_BCM_TX_SETUP: int
    CAN_BCM_TX_DELETE: int
    CAN_BCM_TX_READ: int
    CAN_BCM_TX_SEND: int
    CAN_BCM_RX_SETUP: int
    CAN_BCM_RX_DELETE: int
    CAN_BCM_RX_READ: int
    CAN_BCM_TX_STATUS: int
    CAN_BCM_TX_EXPIRED: int
    CAN_BCM_RX_STATUS: int
    CAN_BCM_RX_TIMEOUT: int
    CAN_BCM_RX_CHANGED: int
    AF_LINK: AddressFamily

if sys.version_info >= (3, 5):
    CAN_RAW_FD_FRAMES: int

if sys.version_info >= (3, 6):
    SO_DOMAIN: int
    SO_PROTOCOL: int
    SO_PEERSEC: int
    SO_PASSSEC: int
    TCP_USER_TIMEOUT: int
    TCP_CONGESTION: int
    AF_ALG: AddressFamily
    SOL_ALG: int
    ALG_SET_KEY: int
    ALG_SET_IV: int
    ALG_SET_OP: int
    ALG_SET_AEAD_ASSOCLEN: int
    ALG_SET_AEAD_AUTHSIZE: int
    ALG_SET_PUBKEY: int
    ALG_OP_DECRYPT: int
    ALG_OP_ENCRYPT: int
    ALG_OP_SIGN: int
    ALG_OP_VERIFY: int

if sys.platform == 'win32':
    SIO_RCVALL: int
    SIO_KEEPALIVE_VALS: int
    RCVALL_IPLEVEL: int
    RCVALL_MAX: int
    RCVALL_OFF: int
    RCVALL_ON: int
    RCVALL_SOCKETLEVELONLY: int

    if sys.version_info >= (3, 6):
        SIO_LOOPBACK_FAST_PATH: int

# enum versions of above flags py 3.4+
if sys.version_info >= (3, 4):
    from enum import IntEnum

    class AddressFamily(IntEnum):
        AF_UNIX = ...
        AF_INET = ...
        AF_INET6 = ...
        AF_APPLETALK = ...
        AF_ASH = ...
        AF_ATMPVC = ...
        AF_ATMSVC = ...
        AF_AX25 = ...
        AF_BLUETOOTH = ...
        AF_BRIDGE = ...
        AF_DECnet = ...
        AF_ECONET = ...
        AF_IPX = ...
        AF_IRDA = ...
        AF_KEY = ...
        AF_LLC = ...
        AF_NETBEUI = ...
        AF_NETLINK = ...
        AF_NETROM = ...
        AF_PACKET = ...
        AF_PPPOX = ...
        AF_ROSE = ...
        AF_ROUTE = ...
        AF_SECURITY = ...
        AF_SNA = ...
        AF_TIPC = ...
        AF_UNSPEC = ...
        AF_WANPIPE = ...
        AF_X25 = ...
        AF_LINK = ...

    class SocketKind(IntEnum):
        SOCK_STREAM = ...
        SOCK_DGRAM = ...
        SOCK_RAW = ...
        SOCK_RDM = ...
        SOCK_SEQPACKET = ...
        SOCK_CLOEXEC = ...
        SOCK_NONBLOCK = ...
else:
    AddressFamily = int
    SocketKind = int

if sys.version_info >= (3, 6):
    from enum import IntFlag

    class AddressInfo(IntFlag):
        AI_ADDRCONFIG = ...
        AI_ALL = ...
        AI_CANONNAME = ...
        AI_NUMERICHOST = ...
        AI_NUMERICSERV = ...
        AI_PASSIVE = ...
        AI_V4MAPPED = ...

    class MsgFlag(IntFlag):
        MSG_CTRUNC = ...
        MSG_DONTROUTE = ...
        MSG_DONTWAIT = ...
        MSG_EOR = ...
        MSG_OOB = ...
        MSG_PEEK = ...
        MSG_TRUNC = ...
        MSG_WAITALL = ...
else:
    AddressInfo = int
    MsgFlag = int


# ----- exceptions -----
class error(IOError):
    ...

class herror(error):
    def __init__(self, herror: int, string: str) -> None: ...

class gaierror(error):
    def __init__(self, error: int, string: str) -> None: ...

class timeout(error):
    ...


# Addresses can be either tuples of varying lengths (AF_INET, AF_INET6,
# AF_NETLINK, AF_TIPC) or strings (AF_UNIX).

# TODO AF_PACKET and AF_BLUETOOTH address objects

_CMSG = Tuple[int, int, bytes]
_SelfT = TypeVar('_SelfT', bound=socket)

# ----- classes -----
class socket:
    family: int
    type: int
    proto: int

    if sys.version_info < (3,):
        def __init__(self, family: int = ..., type: int = ...,
                 proto: int = ...) -> None: ...
    else:
        def __init__(self, family: int = ..., type: int = ...,
                     proto: int = ..., fileno: Optional[int] = ...) -> None: ...

    if sys.version_info >= (3, 2):
        def __enter__(self: _SelfT) -> _SelfT: ...
        def __exit__(self, *args: Any) -> None: ...

    # --- methods ---
    # second tuple item is an address
    def accept(self) -> Tuple[socket, Any]: ...
    def bind(self, address: Union[tuple, str, bytes]) -> None: ...
    def close(self) -> None: ...
    def connect(self, address: Union[tuple, str, bytes]) -> None: ...
    def connect_ex(self, address: Union[tuple, str, bytes]) -> int: ...
    def detach(self) -> int: ...
    def fileno(self) -> int: ...

    # return value is an address
    def getpeername(self) -> Any: ...
    def getsockname(self) -> Any: ...

    @overload
    def getsockopt(self, level: int, optname: int) -> int: ...
    @overload
    def getsockopt(self, level: int, optname: int, buflen: int) -> bytes: ...

    def gettimeout(self) -> float: ...
    def ioctl(self, control: object,
              option: Tuple[int, int, int]) -> None: ...
    def listen(self, backlog: int) -> None: ...
    # TODO the return value may be BinaryIO or TextIO, depending on mode
    def makefile(self, mode: str = ..., buffering: int = ...,
                 encoding: str = ..., errors: str = ...,
                 newline: str = ...) -> Any:
        ...
    def recv(self, bufsize: int, flags: int = ...) -> bytes: ...

    # return type is an address
    def recvfrom(self, bufsize: int, flags: int = ...) -> Any: ...
    def recvfrom_into(self, buffer: _WriteBuffer, nbytes: int,
                      flags: int = ...) -> Any: ...
    def recv_into(self, buffer: _WriteBuffer, nbytes: int,
                  flags: int = ...) -> Any: ...
    def send(self, data: bytes, flags: int = ...) -> int: ...
    def sendall(self, data: bytes, flags: int =...) -> None:
        ...  # return type: None on success
    @overload
    def sendto(self, data: bytes, address: Union[tuple, str]) -> int: ...
    @overload
    def sendto(self, data: bytes, flags: int, address: Union[tuple, str]) -> int: ...
    def setblocking(self, flag: bool) -> None: ...
    def settimeout(self, value: Optional[float]) -> None: ...
    def setsockopt(self, level: int, optname: int, value: Union[int, bytes]) -> None: ...
    def shutdown(self, how: int) -> None: ...

    if sys.version_info >= (3, 3):
        def recvmsg(self, __bufsize: int, __ancbufsize: int = ...,
                    __flags: int = ...) -> Tuple[bytes, List[_CMSG], int, Any]: ...
        def recvmsg_into(self, __buffers: Iterable[_WriteBuffer], __ancbufsize: int = ...,
                         __flags: int = ...) -> Tuple[int, List[_CMSG], int, Any]: ...
        def sendmsg(self, __buffers: Iterable[bytes], __ancdata: Iterable[_CMSG] = ...,
                    __flags: int = ..., __address: Any = ...) -> int: ...


# ----- functions -----
def create_connection(address: Tuple[Optional[str], int],
                      timeout: float = ...,
                      source_address: Tuple[str, int] = ...) -> socket: ...

# the 5th tuple item is an address
# TODO the "Tuple[Any, ...]" should be "Union[Tuple[str, int], Tuple[str, int, int, int]]" but that triggers
# https://github.com/python/mypy/issues/2509
def getaddrinfo(
        host: Optional[str], port: Union[str, int, None], family: int = ...,
        socktype: int = ..., proto: int = ...,
        flags: int = ...) -> List[Tuple[int, int, int, str, Tuple[Any, ...]]]:
    ...

def getfqdn(name: str = ...) -> str: ...
def gethostbyname(hostname: str) -> str: ...
def gethostbyname_ex(hostname: str) -> Tuple[str, List[str], List[str]]: ...
def gethostname() -> str: ...
def gethostbyaddr(ip_address: str) -> Tuple[str, List[str], List[str]]: ...
def getnameinfo(sockaddr: tuple, flags: int) -> Tuple[str, int]: ...
def getprotobyname(protocolname: str) -> int: ...
def getservbyname(servicename: str, protocolname: str = ...) -> int: ...
def getservbyport(port: int, protocolname: str = ...) -> str: ...
def socketpair(family: int = ...,
               type: int = ...,
               proto: int = ...) -> Tuple[socket, socket]: ...
def fromfd(fd: int, family: int, type: int, proto: int = ...) -> socket: ...
def ntohl(x: int) -> int: ...  # param & ret val are 32-bit ints
def ntohs(x: int) -> int: ...  # param & ret val are 16-bit ints
def htonl(x: int) -> int: ...  # param & ret val are 32-bit ints
def htons(x: int) -> int: ...  # param & ret val are 16-bit ints
def inet_aton(ip_string: str) -> bytes: ...  # ret val 4 bytes in length
def inet_ntoa(packed_ip: bytes) -> str: ...
def inet_pton(address_family: int, ip_string: str) -> bytes: ...
def inet_ntop(address_family: int, packed_ip: bytes) -> str: ...
def getdefaulttimeout() -> Optional[float]: ...
def setdefaulttimeout(timeout: Optional[float]) -> None: ...

if sys.version_info >= (3, 3):
    def CMSG_LEN(length: int) -> int: ...
    def CMSG_SPACE(length: int) -> int: ...
    def sethostname(name: str) -> None: ...
    def if_nameindex() -> List[Tuple[int, str]]: ...
    def if_nametoindex(name: str) -> int: ...
    def if_indextoname(index: int) -> str: ...



from typing import Any, List, Optional, Tuple, Type, TypeVar

from queue import Queue

families = ...  # type: List[None]

_TConnection = TypeVar('_TConnection', bound=Connection)
_TListener = TypeVar('_TListener', bound=Listener)

class Connection(object):
    _in = ...  # type: Any
    _out = ...  # type: Any
    recv = ...  # type: Any
    recv_bytes = ...  # type: Any
    send = ...  # type: Any
    send_bytes = ...  # type: Any
    def __enter__(self: _TConnection) -> _TConnection: ...
    def __exit__(self, exc_type, exc_value, exc_tb) -> None: ...
    def __init__(self, _in, _out) -> None: ...
    def close(self) -> None: ...
    def poll(self, timeout: float=...) -> bool: ...

class Listener(object):
    _backlog_queue = ...  # type: Optional[Queue]
    @property
    def address(self) -> Optional[Queue]: ...
    def __enter__(self: _TListener) -> _TListener: ...
    def __exit__(self, exc_type, exc_value, exc_tb) -> None: ...
    def __init__(self, address=..., family=..., backlog=...) -> None: ...
    def accept(self) -> Connection: ...
    def close(self) -> None: ...


def Client(address) -> Connection: ...
def Pipe(duplex: bool=...) -> Tuple[Connection, Connection]: ...



# Stubs for email.message (Python 3.4)

from typing import (
    List, Optional, Union, Tuple, TypeVar, Generator, Sequence, Iterator, Any
)
import sys
from email.charset import Charset
from email.errors import MessageDefect
from email.header import Header
from email.policy import Policy
from email.contentmanager import ContentManager

_T = TypeVar('_T')

_PayloadType = Union[List[Message], str, bytes]
_CharsetType = Union[Charset, str, None]
_ParamsType = Union[str, None, Tuple[str, Optional[str], str]]
_ParamType = Union[str, Tuple[Optional[str], Optional[str], str]]
_HeaderType = Union[str, Header]

class Message:
    preamble = ...  # type: Optional[str]
    epilogue = ...  # type: Optional[str]
    defects = ...  # type: List[MessageDefect]
    def __str__(self) -> str: ...
    def is_multipart(self) -> bool: ...
    def set_unixfrom(self, unixfrom: str) -> None: ...
    def get_unixfrom(self) -> Optional[str]: ...
    def attach(self, payload: Message) -> None: ...
    def get_payload(self, i: int = ..., decode: bool = ...) -> Optional[_PayloadType]: ...
    def set_payload(self, payload: _PayloadType,
                    charset: _CharsetType = ...) -> None: ...
    def set_charset(self, charset: _CharsetType) -> None: ...
    def get_charset(self) -> _CharsetType: ...
    def __len__(self) -> int: ...
    def __contains__(self, name: str) -> bool: ...
    def __getitem__(self, name: str) -> Optional[_HeaderType]: ...
    def __setitem__(self, name: str, val: _HeaderType) -> None: ...
    def __delitem__(self, name: str) -> None: ...
    def keys(self) -> List[str]: ...
    def values(self) -> List[_HeaderType]: ...
    def items(self) -> List[Tuple[str, _HeaderType]]: ...
    def get(self, name: str, failobj: _T = ...) -> Union[_HeaderType, _T]: ...
    def get_all(self, name: str, failobj: _T = ...) -> Union[List[_HeaderType], _T]: ...
    def add_header(self, _name: str, _value: str, **_params: _ParamsType) -> None: ...
    def replace_header(self, _name: str, _value: _HeaderType) -> None: ...
    def get_content_type(self) -> str: ...
    def get_content_maintype(self) -> str: ...
    def get_content_subtype(self) -> str: ...
    def get_default_type(self) -> str: ...
    def set_default_type(self, ctype: str) -> None: ...
    def get_params(self, failobj: _T = ..., header: str = ...,
                   unquote: bool = ...) -> Union[List[Tuple[str, str]], _T]: ...
    def get_param(self, param: str, failobj: _T = ..., header: str = ...,
                  unquote: bool = ...) -> Union[_T, _ParamType]: ...
    def del_param(self, param: str, header: str = ...,
                  requote: bool = ...) -> None: ...
    def set_type(self, type: str, header: str = ...,
                 requote: bool = ...) -> None: ...
    def get_filename(self, failobj: _T = ...) -> Union[_T, str]: ...
    def get_boundary(self, failobj: _T = ...) -> Union[_T, str]: ...
    def set_boundary(self, boundary: str) -> None: ...
    def get_content_charset(self, failobj: _T = ...) -> Union[_T, str]: ...
    def get_charsets(self, failobj: _T = ...) -> Union[_T, List[str]]: ...
    def walk(self) -> Generator[Message, None, None]: ...
    if sys.version_info >= (3, 5):
        def get_content_disposition(self) -> Optional[str]: ...
    def as_string(self, unixfrom: bool = ..., maxheaderlen: int = ...,
                  policy: Optional[Policy] = ...) -> str: ...
    def as_bytes(self, unixfrom: bool = ...,
                 policy: Optional[Policy] = ...) -> bytes: ...
    def __bytes__(self) -> bytes: ...
    def set_param(self, param: str, value: str, header: str = ...,
                  requote: bool = ..., charset: str = ...,
                  language: str = ..., replace: bool = ...) -> None: ...
    def __init__(self, policy: Policy = ...) -> None: ...

class MIMEPart(Message):
    def get_body(self,
                 preferencelist: Sequence[str] = ...) -> Optional[Message]: ...
    def iter_attachments(self) -> Iterator[Message]: ...
    def iter_parts(self) -> Iterator[Message]: ...
    def get_content(self, *args: Any,
                    content_manager: Optional[ContentManager] = ...,
                    **kw: Any) -> Any: ...
    def set_content(self, *args: Any,
                    content_manager: Optional[ContentManager] = ...,
                    **kw: Any) -> None: ...
    def make_related(self, boundary: Optional[str] = ...) -> None: ...
    def make_alternative(self, boundary: Optional[str] = ...) -> None: ...
    def make_mixed(self, boundary: Optional[str] = ...) -> None: ...
    def add_related(self, *args: Any,
                    content_manager: Optional[ContentManager] = ...,
                    **kw: Any) -> None: ...
    def add_alternative(self, *args: Any,
                        content_manager: Optional[ContentManager] = ...,
                        **kw: Any) -> None: ...
    def add_attachement(self, *args: Any,
                        content_manager: Optional[ContentManager] = ...,
                        **kw: Any) -> None: ...
    def clear(self) -> None: ...
    def clear_content(self) -> None: ...
    def is_attachment(self) -> bool: ...

class EmailMessage(MIMEPart): ...



from typing import Any, Optional

class User:
    type = ...  # type: Any
    id = ...  # type: Any
    display_name = ...  # type: Any
    def __init__(self, parent: Optional[Any] = ..., id: str = ..., display_name: str = ...) -> None: ...
    def startElement(self, name, attrs, connection): ...
    def endElement(self, name, value, connection): ...
    def to_xml(self, element_name: str = ...): ...



# Stubs for colorsys

from typing import Tuple

def rgb_to_yiq(r: float, g: float, b: float) -> Tuple[float, float, float]: ...
def yiq_to_rgb(y: float, i: float, q: float) -> Tuple[float, float, float]: ...
def rgb_to_hls(r: float, g: float, b: float) -> Tuple[float, float, float]: ...
def hls_to_rgb(h: float, l: float, s: float) -> Tuple[float, float, float]: ...
def rgb_to_hsv(r: float, g: float, b: float) -> Tuple[float, float, float]: ...
def hsv_to_rgb(h: float, s: float, v: float) -> Tuple[float, float, float]: ...

# TODO undocumented
ONE_SIXTH = ...  # type: float
ONE_THIRD = ...  # type: float
TWO_THIRD = ...  # type: float



from typing import Any, Optional

class StrongRandom:
    def __init__(self, rng: Optional[Any] = ..., randfunc: Optional[Any] = ...) -> None: ...
    def getrandbits(self, k): ...
    def randrange(self, *args): ...
    def randint(self, a, b): ...
    def choice(self, seq): ...
    def shuffle(self, x): ...
    def sample(self, population, k): ...

getrandbits = ...  # type: Any
randrange = ...  # type: Any
randint = ...  # type: Any
choice = ...  # type: Any
shuffle = ...  # type: Any
sample = ...  # type: Any



from typing import Any
from tornado.util import ObjectDict
from collections import namedtuple

class SSLError(Exception): ...

class _NormalizedHeaderCache(dict):
    size = ...  # type: Any
    queue = ...  # type: Any
    def __init__(self, size) -> None: ...
    def __missing__(self, key): ...

class HTTPHeaders(dict):
    def __init__(self, *args, **kwargs) -> None: ...
    def add(self, name, value): ...
    def get_list(self, name): ...
    def get_all(self): ...
    def parse_line(self, line): ...
    @classmethod
    def parse(cls, headers): ...
    def __setitem__(self, name, value): ...
    def __getitem__(self, name): ...
    def __delitem__(self, name): ...
    def __contains__(self, name): ...
    def get(self, name, default=...): ...
    def update(self, *args, **kwargs): ...
    def copy(self): ...
    __copy__ = ...  # type: Any
    def __deepcopy__(self, memo_dict): ...

class HTTPServerRequest:
    method = ...  # type: Any
    uri = ...  # type: Any
    version = ...  # type: Any
    headers = ...  # type: Any
    body = ...  # type: Any
    remote_ip = ...  # type: Any
    protocol = ...  # type: Any
    host = ...  # type: Any
    files = ...  # type: Any
    connection = ...  # type: Any
    arguments = ...  # type: Any
    query_arguments = ...  # type: Any
    body_arguments = ...  # type: Any
    def __init__(self, method=..., uri=..., version=..., headers=..., body=..., host=..., files=..., connection=..., start_line=...) -> None: ...
    def supports_http_1_1(self): ...
    @property
    def cookies(self): ...
    def write(self, chunk, callback=...): ...
    def finish(self): ...
    def full_url(self): ...
    def request_time(self): ...
    def get_ssl_certificate(self, binary_form=...): ...

class HTTPInputError(Exception): ...
class HTTPOutputError(Exception): ...

class HTTPServerConnectionDelegate:
    def start_request(self, server_conn, request_conn): ...
    def on_close(self, server_conn): ...

class HTTPMessageDelegate:
    def headers_received(self, start_line, headers): ...
    def data_received(self, chunk): ...
    def finish(self): ...
    def on_connection_close(self): ...

class HTTPConnection:
    def write_headers(self, start_line, headers, chunk=..., callback=...): ...
    def write(self, chunk, callback=...): ...
    def finish(self): ...

def url_concat(url, args): ...

class HTTPFile(ObjectDict): ...

def parse_body_arguments(content_type, body, arguments, files, headers=...): ...
def parse_multipart_form_data(boundary, data, arguments, files): ...
def format_timestamp(ts): ...

RequestStartLine = namedtuple('RequestStartLine', ['method', 'path', 'version'])

def parse_request_start_line(line): ...

ResponseStartLine = namedtuple('ResponseStartLine', ['version', 'code', 'reason'])

def parse_response_start_line(line): ...
def doctests(): ...
def split_host_and_port(netloc): ...
